@@ -3,6 +3,7 @@ from . import transforms
 
 from .continuous import Uniform
 from .continuous import Flat
+from .continuous import HalfFlat
 from .continuous import Normal
 from .continuous import Beta
 from .continuous import Exponential
@@ -25,6 +26,7 @@ from .continuous import VonMises
 from .continuous import SkewNormal
 from .continuous import Triangular
 from .continuous import Gumbel
+from .continuous import Logistic
 from .continuous import Interpolated
 
 from .discrete import Binomial
@@ -42,7 +44,6 @@ from .discrete import DiscreteUniform
 from .discrete import Geometric
 from .discrete import Categorical
 
-from .distribution import Bound
 from .distribution import DensityDist
 from .distribution import Distribution
 from .distribution import Continuous
@@ -65,6 +66,7 @@ from .multivariate import LKJCholeskyCov
 from .multivariate import LKJCorr
 
 from .timeseries import AR1
+from .timeseries import AR
 from .timeseries import GaussianRandomWalk
 from .timeseries import GARCH11
 from .timeseries import MvGaussianRandomWalk
@@ -76,8 +78,11 @@ from .transforms import logodds
 from .transforms import log
 from .transforms import sum_to_1
 
+from .bound import Bound
+
 __all__ = ['Uniform',
            'Flat',
+           'HalfFlat',
            'Normal',
            'Beta',
            'Exponential',
@@ -126,6 +131,7 @@ __all__ = ['Uniform',
            'LKJCholeskyCov',
            'LKJCorr',
            'AR1',
+           'AR',
            'GaussianRandomWalk',
            'MvGaussianRandomWalk',
            'MvStudentTRandomWalk',
@@ -136,5 +142,7 @@ __all__ = ['Uniform',
            'Triangular',
            'DiscreteWeibull',
            'Gumbel',
-           'Interpolated'
+           'Logistic',
+           'Interpolated',
+           'Bound',
            ]
