@@ -21,13 +21,14 @@ from .dist_math import (
     bound, logpow, gammaln, betaln, std_cdf, i0,
     i1, alltrue_elemwise, SplineWrapper
 )
-from .distribution import Continuous, draw_values, generate_samples, Bound
+from .distribution import Continuous, draw_values, generate_samples
+from .bound import Bound
 
-__all__ = ['Uniform', 'Flat', 'Normal', 'Beta', 'Exponential', 'Laplace',
-           'StudentT', 'Cauchy', 'HalfCauchy', 'Gamma', 'Weibull',
+__all__ = ['Uniform', 'Flat', 'HalfFlat', 'Normal', 'Beta', 'Exponential',
+           'Laplace', 'StudentT', 'Cauchy', 'HalfCauchy', 'Gamma', 'Weibull',
            'HalfStudentT', 'StudentTpos', 'Lognormal', 'ChiSquared',
            'HalfNormal', 'Wald', 'Pareto', 'InverseGamma', 'ExGaussian',
-           'VonMises', 'SkewNormal', 'Interpolated']
+           'VonMises', 'SkewNormal', 'Logistic', 'Interpolated']
 
 
 class PositiveContinuous(Continuous):
@@ -116,6 +117,24 @@ class Uniform(Continuous):
 
        f(x \mid lower, upper) = \frac{1}{upper-lower}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        x = np.linspace(-3.0, 3.0, 1000)
+        a, b = 0.0, 2.0
+        y = np.zeros(1000)
+        y[(x<b) & (x>a)] = 1.0/(b-a)
+        fig, ax = plt.subplots()
+        ax.plot(x, y, label='lower=1, upper=2')
+        a, b = -2.0, 1.0
+        y = np.zeros(1000)
+        y[(x<b) & (x>a)] = 1.0/(b-a)
+        ax.plot(x, y, label='lower=-2, upper=1')
+        ax.legend(loc='upper right')
+        ax.set(ylim=[-0.2,1.2], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  =====================================
     Support   :math:`x \in [lower, upper]`
     Mean      :math:`\dfrac{lower + upper}{2}`
@@ -136,12 +155,11 @@ class Uniform(Continuous):
             transform = transforms.interval(lower, upper)
         super(Uniform, self).__init__(transform=transform, *args, **kwargs)
 
-        self.lower = lower = tt.as_tensor_variable(lower)
-        self.upper = upper = tt.as_tensor_variable(upper)
+        self.lower = lower = floatX(tt.as_tensor_variable(lower))
+        self.upper = upper = floatX(tt.as_tensor_variable(upper))
         self.mean = (upper + lower) / 2.
         self.median = self.mean
 
-
     def random(self, point=None, size=None, repeat=None):
         lower, upper = draw_values([self.lower, self.upper],
                                    point=point)
@@ -173,8 +191,8 @@ class Flat(Continuous):
     """
 
     def __init__(self, *args, **kwargs):
-        super(Flat, self).__init__(*args, **kwargs)
-        self.median = 0
+        self._default = 0
+        super(Flat, self).__init__(defaults=('_default',), *args, **kwargs)
 
     def random(self, point=None, size=None, repeat=None):
         raise ValueError('Cannot sample from Flat distribution')
@@ -183,9 +201,24 @@ class Flat(Continuous):
         return tt.zeros_like(value)
 
     def _repr_latex_(self, name=None, dist=None):
-        if dist is None:
-            dist = self
-        return r'${} \sim \text{{Flat}()$'
+        return r'${} \sim \text{{Flat}}()$'.format(name)
+
+
+class HalfFlat(PositiveContinuous):
+    """Improper flat prior over the positive reals."""
+
+    def __init__(self, *args, **kwargs):
+        self._default = 1
+        super(HalfFlat, self).__init__(defaults=('_default',), *args, **kwargs)
+
+    def random(self, point=None, size=None, repeat=None):
+        raise ValueError('Cannot sample from HalfFlat distribution')
+
+    def logp(self, value):
+        return bound(tt.zeros_like(value), value > 0)
+
+    def _repr_latex_(self, name=None, dist=None):
+        return r'${} \sim \text{{HalfFlat}}()$'.format(name)
 
 
 class Normal(Continuous):
@@ -212,6 +245,23 @@ class Normal(Continuous):
 
        \tau = \dfrac{1}{\sigma^2}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-5.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, sd : st.norm.pdf(x, loc=mu, scale=sd)
+        plot_pdf = lambda a, b : ax.plot(x, f(a,b), label=r'$\mu$={0}, $\sigma$={1}'.format(a,b))
+        plot_pdf(0.0, 0.4)
+        plot_pdf(0.0, 1.0)
+        plot_pdf(0.0, 2.0)
+        plot_pdf(-2.0, 0.4)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-5,5], ylim=[0,1.2], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     Parameters
     ----------
     mu : float
@@ -270,6 +320,22 @@ class HalfNormal(PositiveContinuous):
            \sqrt{\frac{2\tau}{\pi}}
            \exp\left\{ {\frac{-x^2 \tau}{2}}\right\}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda b : st.halfnorm.pdf(x, scale=1.0/np.sqrt(b))
+        plot_pdf = lambda b : ax.plot(x, f(b), label=r'$\tau$={0}'.format(b))
+        plot_pdf(0.5)
+        plot_pdf(1.0)
+        plot_pdf(2.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,5], ylim=[0,1.2], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ==========================================
     Support   :math:`x \in [0, \infty)`
     Mean      :math:`0`
@@ -317,6 +383,7 @@ class HalfNormal(PositiveContinuous):
         return r'${} \sim \text{{HalfNormal}}(\mathit{{sd}}={})$'.format(name,
                                                                 get_variable_name(sd))
 
+
 class Wald(PositiveContinuous):
     R"""
     Wald log-likelihood.
@@ -329,6 +396,24 @@ class Wald(PositiveContinuous):
                -\frac{\lambda}{2x}\left(\frac{x-\mu}{\mu}\right)^2
            \right\}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 3.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, lam : st.invgauss.pdf(x, mu/lam, scale=lam)
+        plot_pdf = lambda a, b : ax.plot(x, f(a,b), label=r'$\mu$={0}, $\lambda$={1}'.format(a,b))
+        plot_pdf(1.0,1.0)
+        plot_pdf(1.0,0.2)
+        plot_pdf(1.0,3.0)
+        plot_pdf(3,1)
+        plot_pdf(3,0.2)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,3], ylim=[0,3.0], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  =============================
     Support   :math:`x \in (0, \infty)`
     Mean      :math:`\mu`
@@ -458,6 +543,24 @@ class Beta(UnitContinuous):
        f(x \mid \alpha, \beta) =
            \frac{x^{\alpha - 1} (1 - x)^{\beta - 1}}{B(\alpha, \beta)}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 1.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda a, b : st.beta.pdf(x, a, b)
+        plot_pdf = lambda a, b : ax.plot(x, f(a,b), label=r'$\alpha$={0}, $\beta$={1}'.format(a,b))
+        plot_pdf(0.5, 0.5)
+        plot_pdf(5.0, 1.0)
+        plot_pdf(1.0, 3.0)
+        plot_pdf(2.0, 2.0)
+        plot_pdf(2.0, 5.0)
+        plt.legend(loc='upper center', frameon=False)
+        ax.set(xlim=[0,1], ylim=[0,2.5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ==============================================================
     Support   :math:`x \in (0, 1)`
     Mean      :math:`\dfrac{\alpha}{\alpha + \beta}`
@@ -484,7 +587,7 @@ class Beta(UnitContinuous):
     mu : float
         Alternative mean (0 < mu < 1).
     sd : float
-        Alternative standard deviation (sd > 0).
+        Alternative standard deviation (0 < sd < sqrt(mu * (1 - mu))).
 
     Notes
     -----
@@ -531,8 +634,13 @@ class Beta(UnitContinuous):
         alpha = self.alpha
         beta = self.beta
 
-        return bound(logpow(value, alpha - 1) + logpow(1 - value, beta - 1)
-                     - betaln(alpha, beta),
+        logval = tt.log(value)
+        log1pval = tt.log1p(-value)
+        logp = (tt.switch(tt.eq(alpha, 1), 0, (alpha - 1) * logval)
+                + tt.switch(tt.eq(beta, 1), 0, (beta - 1) * log1pval)
+                - betaln(alpha, beta))
+
+        return bound(logp,
                      value >= 0, value <= 1,
                      alpha > 0, beta > 0)
 
@@ -554,6 +662,22 @@ class Exponential(PositiveContinuous):
 
        f(x \mid \lambda) = \lambda \exp\left\{ -\lambda x \right\}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda lam : st.expon.pdf(x, scale=1.0/lam)
+        plot_pdf = lambda lam : ax.plot(x, f(lam), label=r'$\lambda$={0}'.format(lam))
+        plot_pdf(0.5)
+        plot_pdf(1.0)
+        plot_pdf(1.5)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,5], ylim=[0,1.6], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ============================
     Support   :math:`x \in [0, \infty)`
     Mean      :math:`\dfrac{1}{\lambda}`
@@ -603,6 +727,23 @@ class Laplace(Continuous):
        f(x \mid \mu, b) =
            \frac{1}{2b} \exp \left\{ - \frac{|x - \mu|}{b} \right\}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-10.0, 10.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, b : st.laplace.pdf(x, loc=mu, scale=b)
+        plot_pdf = lambda mu, b : ax.plot(x, f(mu, b), label=r'$\mu$={0}, $b$={1}'.format(mu, b))
+        plot_pdf(0.0, 1.0)
+        plot_pdf(0.0, 2.0)
+        plot_pdf(0.0, 4)
+        plot_pdf(-5.0, 4)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-10,10], ylim=[0,0.5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ========================
     Support   :math:`x \in \mathbb{R}`
     Mean      :math:`\mu`
@@ -663,10 +804,26 @@ class Lognormal(PositiveContinuous):
            \frac{1}{x} \sqrt{\frac{\tau}{2\pi}}
            \exp\left\{ -\frac{\tau}{2} (\ln(x)-\mu)^2 \right\}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 3.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, sd : st.lognorm.pdf(x, sd, scale=np.exp(mu))
+        plot_pdf = lambda mu, sd : ax.plot(x, f(mu, sd), label=r'$\mu$={0}, $\sigma$={1}'.format(mu, sd))
+        plot_pdf(0.0, 0.25)
+        plot_pdf(0.0, 0.5)
+        plot_pdf(0.0, 1.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,3], ylim=[0,1.8], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  =========================================================================
-    Support   :math:`x \in (0, 1)`
+    Support   :math:`x \in [0, \infty)`
     Mean      :math:`\exp\{\mu + \frac{1}{2\tau}\}`
-    Variance  :math:\(\exp\{\frac{1}{\tau}\} - 1\) \times \exp\{2\mu + \frac{1}{\tau}\}
+    Variance  :math:`(\exp\{\frac{1}{\tau}\} - 1) \times \exp\{2\mu + \frac{1}{\tau}\}`
     ========  =========================================================================
 
     Parameters
@@ -736,14 +893,32 @@ class StudentT(Continuous):
            \left(\frac{\lambda}{\pi\nu}\right)^{\frac{1}{2}}
            \left[1+\frac{\lambda(x-\mu)^2}{\nu}\right]^{-\frac{\nu+1}{2}}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-5.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, lam, df : st.t.pdf(x, df, loc=mu, scale=1.0/np.sqrt(lam))
+        plot_pdf = lambda mu, lam, df : ax.plot(x, f(mu, lam, df), label=r'$\mu$={0}, $\lambda$={1}, $\nu$={2}'.format(mu, lam, df))
+        plot_pdf(0.0, 1.0, 1.0)
+        plot_pdf(0.0, 1.0, 2.0)
+        plot_pdf(0.0, 1.0, 5)
+        plot_pdf(-1.0, 1.0, 5)
+        plot_pdf(-1.0, 2.0, 5)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-5,5], ylim=[0,0.6], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ========================
     Support   :math:`x \in \mathbb{R}`
     ========  ========================
 
     Parameters
     ----------
-    nu : int
-        Degrees of freedom (nu > 0).
+    nu : float
+        Degrees of freedom, also known as normality parameter (nu > 0).
     mu : float
         Location parameter.
     lam : float
@@ -807,6 +982,22 @@ class Pareto(PositiveContinuous):
 
        f(x \mid \alpha, m) = \frac{\alpha m^{\alpha}}{x^{\alpha+1}}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda m, alpha : st.pareto.pdf(x, alpha, scale=m)
+        plot_pdf = lambda m, alpha : ax.plot(x, f(m, alpha), label=r'm={0}, $\alpha$={1}'.format(m, alpha))
+        plot_pdf(1.0,1.0)
+        plot_pdf(1.0,2.0)
+        plot_pdf(1.0,3.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,5], ylim=[0,3.0], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  =============================================================
     Support   :math:`x \in [m, \infty)`
     Mean      :math:`\dfrac{\alpha m}{\alpha - 1}` for :math:`\alpha \ge 1`
@@ -878,6 +1069,23 @@ class Cauchy(Continuous):
        f(x \mid \alpha, \beta) =
            \frac{1}{\pi \beta [1 + (\frac{x-\alpha}{\beta})^2]}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-5.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda a, b : st.cauchy.pdf(x, loc=a, scale=b)
+        plot_pdf = lambda a, b : ax.plot(x, f(a, b), label=r'$\alpha$={0}, $\beta$={1}'.format(a, b))
+        plot_pdf(0.0, 0.5)
+        plot_pdf(0.0, 1.0)
+        plot_pdf(0.0, 2.0)
+        plot_pdf(-2.0, 1.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-5,5], ylim=[0,0.7], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ========================
     Support   :math:`x \in \mathbb{R}`
     Mode      :math:`\alpha`
@@ -936,6 +1144,22 @@ class HalfCauchy(PositiveContinuous):
 
        f(x \mid \beta) = \frac{2}{\pi \beta [1 + (\frac{x}{\beta})^2]}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda b : st.cauchy.pdf(x, scale=b)
+        plot_pdf = lambda b : ax.plot(x, f(b), label=r'$\beta$={0}'.format(b))
+        plot_pdf(0.5)
+        plot_pdf(1.0)
+        plot_pdf(2.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,5], ylim=[0,0.7], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ========================
     Support   :math:`x \in \mathbb{R}`
     Mode      0
@@ -992,6 +1216,23 @@ class Gamma(PositiveContinuous):
        f(x \mid \alpha, \beta) =
            \frac{\beta^{\alpha}x^{\alpha-1}e^{-\beta x}}{\Gamma(\alpha)}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 20.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda a, b : st.gamma.pdf(x, a, scale=1.0/b)
+        plot_pdf = lambda a, b : ax.plot(x, f(a, b), label=r'$\alpha$={0}, $\beta$={1}'.format(a, b))
+        plot_pdf(1.0, 0.5)
+        plot_pdf(2.0, 0.5)
+        plot_pdf(3.0, 1.0)
+        plot_pdf(7.5, 1.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,20], ylim=[0,0.5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ===============================
     Support   :math:`x \in (0, \infty)`
     Mean      :math:`\dfrac{\alpha}{\beta}`
@@ -1082,6 +1323,23 @@ class InverseGamma(PositiveContinuous):
            \frac{\beta^{\alpha}}{\Gamma(\alpha)} x^{-\alpha - 1}
            \exp\left(\frac{-\beta}{x}\right)
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 3.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda alpha, beta : st.invgamma.pdf(x, alpha, scale=beta)
+        plot_pdf = lambda alpha, beta : ax.plot(x, f(alpha, beta), label=r'$\alpha$={0}, $\beta$={1}'.format(alpha, beta))
+        plot_pdf(1.0,1.0)
+        plot_pdf(2.0,1.0)
+        plot_pdf(3.0,1.0)
+        plot_pdf(3.0,0.5)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,3], ylim=[0,5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ======================================================
     Support   :math:`x \in (0, \infty)`
     Mean      :math:`\dfrac{\beta}{\alpha-1}` for :math:`\alpha > 1`
@@ -1150,6 +1408,25 @@ class ChiSquared(Gamma):
 
        f(x \mid \nu) = \frac{x^{(\nu-2)/2}e^{-x/2}}{2^{\nu/2}\Gamma(\nu/2)}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 8.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda df : st.chi2.pdf(x, df)
+        plot_pdf = lambda df : ax.plot(x, f(df), label=r'$\nu$={0}'.format(df))
+        plot_pdf(1.0)
+        plot_pdf(2.0)
+        plot_pdf(3.0)
+        plot_pdf(4.0)
+        plot_pdf(6.0)
+        plot_pdf(9.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,8], ylim=[0,0.5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ===============================
     Support   :math:`x \in [0, \infty)`
     Mean      :math:`\nu`
@@ -1185,6 +1462,23 @@ class Weibull(PositiveContinuous):
            \frac{\alpha x^{\alpha - 1}
            \exp(-(\frac{x}{\beta})^{\alpha})}{\beta^\alpha}
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(0.0, 2.5, 1000)
+        fig, ax = plt.subplots()
+        f = lambda alpha, beta : st.weibull_min.pdf(x, alpha, scale=beta)
+        plot_pdf = lambda alpha, beta : ax.plot(x, f(alpha, beta), label=r'$\alpha$={0}, $\beta$={1}'.format(alpha, beta))
+        plot_pdf(0.5, 1.0)
+        plot_pdf(1.0, 1.0)
+        plot_pdf(1.5, 1.0)
+        plot_pdf(5.0, 1.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[0,2.5], ylim=[0,2.5], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ====================================================
     Support   :math:`x \in [0, \infty)`
     Mean      :math:`\beta \Gamma(1 + \frac{1}{\alpha})`
@@ -1265,6 +1559,23 @@ class ExGaussian(Continuous):
     where :math:`\Phi` is the cumulative distribution function of the
     standard normal distribution.
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-6.0, 6.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, sigma, nu : st.exponnorm.pdf(x, nu/sigma, loc=mu, scale=sigma)
+        plot_pdf = lambda mu, sigma, nu : ax.plot(x, f(mu, sigma, nu), label=r'$\mu$={0}, $\sigma$={1}, $\nu$={2}'.format(mu, sigma, nu))
+        plot_pdf(0.0,1.0,1.0)
+        plot_pdf(-2.0,1.0,1.0)
+        plot_pdf(0.0,3.0,1.0)
+        plot_pdf(-3.0,1.0,4.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-6,6], ylim=[0,0.4], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ========================
     Support   :math:`x \in \mathbb{R}`
     Mean      :math:`\mu + \nu`
@@ -1349,7 +1660,25 @@ class VonMises(Continuous):
         f(x \mid \mu, \kappa) =
             \frac{e^{\kappa\cos(x-\mu)}}{2\pi I_0(\kappa)}
 
-    where :I_0 is the modified Bessel function of order 0.
+    where :math:`I_0` is the modified Bessel function of order 0.
+
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-np.pi, np.pi, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, kappa : st.vonmises.pdf(x, kappa, loc=mu)
+        plot_pdf = lambda mu, kappa : ax.plot(x, f(mu, kappa), label=r'$\mu$={0}, $\kappa$={1}'.format(mu, kappa))
+        plot_pdf(0.0,0.001)
+        plot_pdf(0.0,0.5)
+        plot_pdf(0.0,1.0)
+        plot_pdf(0.0,2.0)
+        plot_pdf(0.0,4.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-np.pi,np.pi], ylim=[0,1.0], xlabel='x', ylabel='f(x)')
+        plt.show()
 
     ========  ==========================================
     Support   :math:`x \in [-\pi, \pi]`
@@ -1407,6 +1736,24 @@ class SkewNormal(Continuous):
        f(x \mid \mu, \tau, \alpha) =
        2 \Phi((x-\mu)\sqrt{\tau}\alpha) \phi(x,\mu,\tau)
 
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-3.0, 3.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda alpha, mu, sigma : st.skewnorm.pdf(x, alpha, loc=mu, scale=sigma)
+        plot_pdf = lambda alpha, mu, sigma : ax.plot(x, f(alpha, mu, sigma), label=r'$\mu$={0}, $\sigma$={1}, $\alpha$={2}'.format(mu, sigma, alpha))
+        plot_pdf(-4.0,0.0,1.0)
+        plot_pdf(-1.0,0.0,1.0)
+        plot_pdf(0.0,0.0,1.0)
+        plot_pdf(1.0,0.0,1.0)
+        plot_pdf(4.0,0.0,1.0)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-3,3], ylim=[0,0.7], xlabel='x', ylabel='f(x)')
+        plt.show()
+
     ========  ==========================================
     Support   :math:`x \in \mathbb{R}`
     Mean      :math:`\mu + \sigma \sqrt{\frac{2}{\pi}} \frac {\alpha }{{\sqrt {1+\alpha ^{2}}}}`
@@ -1589,6 +1936,83 @@ class Gumbel(Continuous):
                                                                 get_variable_name(beta))
 
 
+class Logistic(Continuous):
+    R"""
+    Logistic log-likelihood.
+
+    .. math::
+
+       f(x \mid \mu, s) =
+           \frac{\exp\left(-\frac{x - \mu}{s}\right)}{s \left(1 + \exp\left(-\frac{x - \mu}{s}\right)\right)^2}
+
+    ========  ==========================================
+    Support   :math:`x \in \mathbb{R}`
+    Mean      :math:`\mu`
+    Variance  :math:`\frac{s^2 \pi^2}{3}`
+    ========  ==========================================
+
+    .. plot::
+
+        import matplotlib.pyplot as plt
+        import numpy as np
+        import scipy.stats as st
+        x = np.linspace(-5.0, 5.0, 1000)
+        fig, ax = plt.subplots()
+        f = lambda mu, s : st.logistic.pdf(x, loc=mu, scale=s)
+        plot_pdf = lambda a, b : ax.plot(x, f(a,b), label=r'$\mu$={0}, $s$={1}'.format(a,b))
+        plot_pdf(0.0, 0.4)
+        plot_pdf(0.0, 1.0)
+        plot_pdf(0.0, 2.0)
+        plot_pdf(-2.0, 0.4)
+        plt.legend(loc='upper right', frameon=False)
+        ax.set(xlim=[-5,5], ylim=[0,1.2], xlabel='x', ylabel='f(x)')
+        plt.show()
+
+    Parameters
+    ----------
+    mu : float
+        Mean.
+    s : float
+        Scale (s > 0).
+    """
+    def __init__(self, mu=0., s=1., *args, **kwargs):
+        super(Logistic, self).__init__(*args, **kwargs)
+
+        self.mu = tt.as_tensor_variable(mu)
+        self.s = tt.as_tensor_variable(s)
+
+        self.mean = self.mode = mu
+        self.variance = s**2 * np.pi**2 / 3.
+
+    def logp(self, value):
+        mu = self.mu
+        s = self.s
+
+        return bound(
+            -(value - mu) / s - tt.log(s) - 2 * tt.log1p(tt.exp(-(value - mu) / s)),
+            s > 0
+        )
+
+    def random(self, point=None, size=None, repeat=None):
+        mu, s = draw_values([self.mu, self.s], point=point)
+
+        return generate_samples(
+            stats.logistic.rvs,
+            loc=mu, scale=s,
+            dist_shape=self.shape,
+            size=size
+        )
+
+    def _repr_latex_(self, name=None, dist=None):
+        if dist is None:
+            dist = self
+        mu = dist.mu
+        s = dist.s
+        return r'${} \sim \text{{Logistic}}(\mathit{{mu}}={}, \mathit{{s}}={})$'.format(name,
+                                                                get_variable_name(mu),
+                                                                get_variable_name(s))
+
+
 class Interpolated(Continuous):
     R"""
     Univariate probability distribution defined as a linear interpolation
