@@ -6,12 +6,9 @@ import theano
 from ..memoize import memoize
 from ..model import Model, get_named_nodes, FreeRV, ObservedRV
 from ..vartypes import string_types
-from .dist_math import bound
-# To avoid circular import for transform below
-import pymc3 as pm
 
-__all__ = ['DensityDist', 'Distribution', 'Continuous', 'Bound',
-           'Discrete', 'NoDistribution', 'TensorType', 'draw_values']
+__all__ = ['DensityDist', 'Distribution', 'Continuous', 'Discrete',
+           'NoDistribution', 'TensorType', 'draw_values']
 
 
 class _Unpickling(object):
@@ -27,8 +24,9 @@ class Distribution(object):
             model = Model.get_context()
         except TypeError:
             raise TypeError("No model on context stack, which is needed to "
-                            "use the Normal('x', 0,1) syntax. "
-                            "Add a 'with model:' block")
+                            "instantiate distributions. Add variable inside "
+                            "a 'with model:' block, or use the '.dist' syntax "
+                            "for a standalone distribution.")
 
         if isinstance(name, string_types):
             data = kwargs.pop('observed', None)
@@ -90,8 +88,33 @@ class Distribution(object):
         return val
 
     def _repr_latex_(self, name=None, dist=None):
+        """Magic method name for IPython to use for LaTeX formatting."""
         return None
 
+    def logp_nojac(self, *args, **kwargs):
+        """Return the logp, but do not include a jacobian term for transforms.
+
+        If we use different parametrizations for the same distribution, we
+        need to add the determinant of the jacobian of the transformation
+        to make sure the densities still describe the same distribution.
+        However, MAP estimates are not invariant with respect to the
+        parametrization, we need to exclude the jacobian terms in this case.
+
+        This function should be overwritten in base classes for transformed
+        distributions.
+        """
+        return self.logp(*args, **kwargs)
+
+    def logp_sum(self, *args, **kwargs):
+        """Return the sum of the logp values for the given observations.
+
+        Subclasses can use this to improve the speed of logp evaluations
+        if only the sum of the logp values is needed.
+        """
+        return tt.sum(self.logp(*args, **kwargs))
+
+    __latex__ = _repr_latex_
+
 
 def TensorType(dtype, shape, broadcastable=None):
     if broadcastable is None:
@@ -109,10 +132,12 @@ class NoDistribution(Distribution):
         self.parent_dist = parent_dist
 
     def __getattr__(self, name):
-        try:
-            self.__dict__[name]
-        except KeyError:
-            return getattr(self.parent_dist, name)
+        # Do not use __getstate__ and __setstate__ from parent_dist
+        # to avoid infinite recursion during unpickling
+        if name.startswith('__'):
+            raise AttributeError(
+                "'NoDistribution' has no attribute '%s'" % name)
+        return getattr(self.parent_dist, name)
 
     def logp(self, x):
         return 0
@@ -161,14 +186,6 @@ class DensityDist(Distribution):
         self.logp = logp
 
 
-class MultivariateContinuous(Continuous):
-    pass
-
-
-class MultivariateDiscrete(Discrete):
-    pass
-
-
 def draw_values(params, point=None):
     """
     Draw (fix) parameter values. Handles a number of cases:
@@ -396,126 +413,3 @@ def generate_samples(generator, *args, **kwargs):
                                         prefix_shape,
                                         *args, **kwargs)
     return reshape_sampled(samples, size, dist_shape)
-
-
-class Bounded(Distribution):
-    R"""
-    An upper, lower or upper+lower bounded distribution
-
-    Parameters
-    ----------
-    distribution : pymc3 distribution
-        Distribution to be transformed into a bounded distribution
-    lower : float (optional)
-        Lower bound of the distribution, set to -inf to disable.
-    upper : float (optional)
-        Upper bound of the distribibution, set to inf to disable.
-    tranform : 'infer' or object
-        If 'infer', infers the right transform to apply from the supplied bounds.
-        If transform object, has to supply .forward() and .backward() methods.
-        See pymc3.distributions.transforms for more information.
-    """
-
-    def __init__(self, distribution, lower, upper, transform='infer', *args, **kwargs):
-        self.dist = distribution.dist(*args, **kwargs)
-
-        self.__dict__.update(self.dist.__dict__)
-        self.__dict__.update(locals())
-
-        if hasattr(self.dist, 'mode'):
-            self.mode = self.dist.mode
-
-        if transform == 'infer':
-
-            default = self.dist.default()
-
-            if not np.isinf(lower) and not np.isinf(upper):
-                self.transform = pm.distributions.transforms.interval(lower, upper)
-                if default <= lower or default >= upper:
-                    self.testval = 0.5 * (upper + lower)
-
-            if not np.isinf(lower) and np.isinf(upper):
-                self.transform = pm.distributions.transforms.lowerbound(lower)
-                if default <= lower:
-                    self.testval = lower + 1
-
-            if np.isinf(lower) and not np.isinf(upper):
-                self.transform = pm.distributions.transforms.upperbound(upper)
-                if default >= upper:
-                    self.testval = upper - 1
-
-        if issubclass(distribution, Discrete):
-            self.transform = None
-
-    def _random(self, lower, upper, point=None, size=None):
-        samples = np.zeros(size).flatten()
-        i, n = 0, len(samples)
-        while i < len(samples):
-            sample = self.dist.random(point=point, size=n)
-            select = sample[np.logical_and(sample > lower, sample <= upper)]
-            samples[i:(i + len(select))] = select[:]
-            i += len(select)
-            n -= len(select)
-        if size is not None:
-            return np.reshape(samples, size)
-        else:
-            return samples
-
-    def random(self, point=None, size=None, repeat=None):
-        lower, upper = draw_values([self.lower, self.upper], point=point)
-        return generate_samples(self._random, lower, upper, point,
-                                dist_shape=self.shape,
-                                size=size)
-
-    def logp(self, value):
-        return bound(self.dist.logp(value),
-                     value >= self.lower, value <= self.upper)
-
-
-class Bound(object):
-    R"""
-    Creates a new upper, lower or upper+lower bounded distribution
-
-    Parameters
-    ----------
-    distribution : pymc3 distribution
-        Distribution to be transformed into a bounded distribution
-    lower : float (optional)
-        Lower bound of the distribution
-    upper : float (optional)
-
-    Example
-    -------
-    # Bounded distribution can be defined before the model context
-    PositiveNormal = pm.Bound(pm.Normal, lower=0.0)
-    with pm.Model():
-        par1 = PositiveNormal('par1', mu=0.0, sd=1.0, testval=1.0)
-        # or within the model context
-        NegativeNormal = pm.Bound(pm.Normal, upper=0.0)
-        par2 = NegativeNormal('par2', mu=0.0, sd=1.0, testval=1.0)
-
-        # or you can define it implicitly within the model context
-        par3 = pm.Bound(pm.Normal, lower=-1.0, upper=1.0)(
-                'par3', mu=0.0, sd=1.0, testval=1.0)
-    """
-
-    def __init__(self, distribution, lower=-np.inf, upper=np.inf):
-        self.distribution = distribution
-        self.lower = lower
-        self.upper = upper
-
-    def __call__(self, *args, **kwargs):
-        if 'observed' in kwargs:
-            raise ValueError('Observed Bound distributions are not allowed. '
-                             'If you want to model truncated data '
-                             'you can use a pm.Potential in combination '
-                             'with the cumulative probability function. See '
-                             'pymc3/examples/censored_data.py for an example.')
-        first, args = args[0], args[1:]
-
-        return Bounded(first, self.distribution, self.lower, self.upper,
-                       *args, **kwargs)
-
-    def dist(self, *args, **kwargs):
-        return Bounded.dist(self.distribution, self.lower, self.upper,
-                            *args, **kwargs)
