@@ -10,10 +10,9 @@ import theano.tensor as tt
 from scipy import stats, linalg
 
 from theano.tensor.nlinalg import det, matrix_inverse, trace
-
+import theano.tensor.slinalg
 import pymc3 as pm
 
-from pymc3.math import tround
 from pymc3.theanof import floatX
 from . import transforms
 from pymc3.util import get_variable_name
@@ -29,7 +28,128 @@ __all__ = ['MvNormal', 'MvStudentT', 'Dirichlet',
            'LKJCorr', 'LKJCholeskyCov']
 
 
-class MvNormal(Continuous):
+class _QuadFormBase(Continuous):
+    def __init__(self, mu=None, cov=None, chol=None, tau=None, lower=True,
+                 *args, **kwargs):
+        super(_QuadFormBase, self).__init__(*args, **kwargs)
+        if len(self.shape) > 2:
+            raise ValueError("Only 1 or 2 dimensions are allowed.")
+
+        if chol is not None and not lower:
+            chol = chol.T
+        if len([i for i in [tau, cov, chol] if i is not None]) != 1:
+            raise ValueError('Incompatible parameterization. '
+                             'Specify exactly one of tau, cov, '
+                             'or chol.')
+        self.mu = mu = tt.as_tensor_variable(mu)
+        self.solve_lower = tt.slinalg.Solve(A_structure="lower_triangular")
+        # Step methods and advi do not catch LinAlgErrors at the
+        # moment. We work around that by using a cholesky op
+        # that returns a nan as first entry instead of raising
+        # an error.
+        cholesky = Cholesky(nofail=True, lower=True)
+
+        if cov is not None:
+            self.k = cov.shape[0]
+            self._cov_type = 'cov'
+            cov = tt.as_tensor_variable(cov)
+            if cov.ndim != 2:
+                raise ValueError('cov must be two dimensional.')
+            self.chol_cov = cholesky(cov)
+            self.cov = cov
+            self._n = self.cov.shape[-1]
+        elif tau is not None:
+            self.k = tau.shape[0]
+            self._cov_type = 'tau'
+            tau = tt.as_tensor_variable(tau)
+            if tau.ndim != 2:
+                raise ValueError('tau must be two dimensional.')
+            self.chol_tau = cholesky(tau)
+            self.tau = tau
+            self._n = self.tau.shape[-1]
+        else:
+            self.k = chol.shape[0]
+            self._cov_type = 'chol'
+            if chol.ndim != 2:
+                raise ValueError('chol must be two dimensional.')
+            self.chol_cov = tt.as_tensor_variable(chol)
+            self._n = self.chol_cov.shape[-1]
+
+    def _quaddist(self, value):
+        """Compute (x - mu).T @ Sigma^-1 @ (x - mu) and the logdet of Sigma."""
+        mu = self.mu
+        if value.ndim > 2 or value.ndim == 0:
+            raise ValueError('Invalid dimension for value: %s' % value.ndim)
+        if value.ndim == 1:
+            onedim = True
+            value = value[None, :]
+        else:
+            onedim = False
+
+        delta = value - mu
+
+        if self._cov_type == 'cov':
+            # Use this when Theano#5908 is released.
+            # return MvNormalLogp()(self.cov, delta)
+            dist, logdet, ok = self._quaddist_cov(delta)
+        elif self._cov_type == 'tau':
+            dist, logdet, ok = self._quaddist_tau(delta)
+        else:
+            dist, logdet, ok = self._quaddist_chol(delta)
+
+        if onedim:
+            return dist[0], logdet, ok
+        return dist, logdet, ok
+
+    def _quaddist_chol(self, delta):
+        chol_cov = self.chol_cov
+        _, k = delta.shape
+        k = pm.floatX(k)
+        diag = tt.nlinalg.diag(chol_cov)
+        # Check if the covariance matrix is positive definite.
+        ok = tt.all(diag > 0)
+        # If not, replace the diagonal. We return -inf later, but
+        # need to prevent solve_lower from throwing an exception.
+        chol_cov = tt.switch(ok, chol_cov, 1)
+
+        delta_trans = self.solve_lower(chol_cov, delta.T).T
+        quaddist = (delta_trans ** 2).sum(axis=-1)
+        logdet = tt.sum(tt.log(diag))
+        return quaddist, logdet, ok
+
+    def _quaddist_cov(self, delta):
+        return self._quaddist_chol(delta)
+
+    def _quaddist_tau(self, delta):
+        chol_tau = self.chol_tau
+        _, k = delta.shape
+        k = pm.floatX(k)
+
+        diag = tt.nlinalg.diag(chol_tau)
+        ok = tt.all(diag > 0)
+
+        chol_tau = tt.switch(ok, chol_tau, 1)
+        diag = tt.nlinalg.diag(chol_tau)
+        delta_trans = tt.dot(delta, chol_tau)
+        quaddist = (delta_trans ** 2).sum(axis=-1)
+        logdet = -tt.sum(tt.log(diag))
+        return quaddist, logdet, ok
+
+    def _repr_cov_params(self, dist=None):
+        if dist is None:
+            dist = self
+        if self._cov_type == 'chol':
+            chol = get_variable_name(self.chol)
+            return r'\mathit{{chol}}={}'.format(chol)
+        elif self._cov_type == 'cov':
+            cov = get_variable_name(self.cov)
+            return r'\mathit{{cov}}={}'.format(cov)
+        elif self._cov_type == 'tau':
+            tau = get_variable_name(self.tau)
+            return r'\mathit{{tau}}={}'.format(tau)
+
+
+class MvNormal(_QuadFormBase):
     R"""
     Multivariate normal log-likelihood.
 
@@ -98,50 +218,9 @@ class MvNormal(Continuous):
 
     def __init__(self, mu, cov=None, tau=None, chol=None, lower=True,
                  *args, **kwargs):
-        super(MvNormal, self).__init__(*args, **kwargs)
-        if len(self.shape) > 2:
-            raise ValueError("Only 1 or 2 dimensions are allowed.")
-
-        if not lower:
-            chol = chol.T
-        if len([i for i in [tau, cov, chol] if i is not None]) != 1:
-            raise ValueError('Incompatible parameterization. '
-                             'Specify exactly one of tau, cov, '
-                             'or chol.')
-        mu = tt.as_tensor_variable(mu)
-        self.mean = self.median = self.mode = self.mu = mu
-        self.solve_lower = tt.slinalg.Solve(A_structure="lower_triangular")
-        # Step methods and advi do not catch LinAlgErrors at the
-        # moment. We work around that by using a cholesky op
-        # that returns a nan as first entry instead of raising
-        # an error.
-        cholesky = Cholesky(nofail=True, lower=True)
-
-        if cov is not None:
-            self.k = cov.shape[0]
-            self._cov_type = 'cov'
-            cov = tt.as_tensor_variable(cov)
-            if cov.ndim != 2:
-                raise ValueError('cov must be two dimensional.')
-            self.chol_cov = cholesky(cov)
-            self.cov = cov
-            self._n = self.cov.shape[-1]
-        elif tau is not None:
-            self.k = tau.shape[0]
-            self._cov_type = 'tau'
-            tau = tt.as_tensor_variable(tau)
-            if tau.ndim != 2:
-                raise ValueError('tau must be two dimensional.')
-            self.chol_tau = cholesky(tau)
-            self.tau = tau
-            self._n = self.tau.shape[-1]
-        else:
-            self.k = chol.shape[0]
-            self._cov_type = 'chol'
-            if chol.ndim != 2:
-                raise ValueError('chol must be two dimensional.')
-            self.chol_cov = tt.as_tensor_variable(chol)
-            self._n = self.chol_cov.shape[-1]
+        super(MvNormal, self).__init__(mu=mu, cov=cov, tau=tau, chol=chol,
+                                       lower=lower, *args, **kwargs)
+        self.mean = self.median = self.mode = self.mu = self.mu
 
     def random(self, point=None, size=None):
         if size is None:
@@ -154,30 +233,30 @@ class MvNormal(Continuous):
 
         if self._cov_type == 'cov':
             mu, cov = draw_values([self.mu, self.cov], point=point)
-            if mu.shape != cov[0].shape:
-                raise ValueError("Shapes for mu an cov don't match")
+            if mu.shape[-1] != cov.shape[-1]:
+                raise ValueError("Shapes for mu and cov don't match")
 
             try:
                 dist = stats.multivariate_normal(
                     mean=mu, cov=cov, allow_singular=True)
-            except ValueError as error:
-                size.append(mu.shape[0])
+            except ValueError:
+                size.append(mu.shape[-1])
                 return np.nan * np.zeros(size)
             return dist.rvs(size)
         elif self._cov_type == 'chol':
             mu, chol = draw_values([self.mu, self.chol_cov], point=point)
-            if mu.shape != chol[0].shape:
-                raise ValueError("Shapes for mu an chol don't match")
+            if mu.shape[-1] != chol[0].shape[-1]:
+                raise ValueError("Shapes for mu and chol don't match")
 
-            size.append(mu.shape[0])
+            size.append(mu.shape[-1])
             standard_normal = np.random.standard_normal(size)
             return mu + np.dot(standard_normal, chol.T)
         else:
             mu, tau = draw_values([self.mu, self.tau], point=point)
-            if mu.shape != tau[0].shape:
-                raise ValueError("Shapes for mu an tau don't match")
+            if mu.shape[-1] != tau[0].shape[-1]:
+                raise ValueError("Shapes for mu and tau don't match")
 
-            size.append(mu.shape[0])
+            size.append(mu.shape[-1])
             standard_normal = np.random.standard_normal(size)
             try:
                 chol = linalg.cholesky(tau, lower=True)
@@ -188,101 +267,22 @@ class MvNormal(Continuous):
             return mu + transformed.T
 
     def logp(self, value):
-        mu = self.mu
-        if value.ndim > 2 or value.ndim == 0:
-            raise ValueError('Invalid dimension for value: %s' % value.ndim)
-        if value.ndim == 1:
-            onedim = True
-            value = value[None, :]
-        else:
-            onedim = False
-
-        delta = value - mu
-
-        if self._cov_type == 'cov':
-            # Use this when Theano#5908 is released.
-            # return MvNormalLogp()(self.cov, delta)
-            logp = self._logp_cov(delta)
-        elif self._cov_type == 'tau':
-            logp = self._logp_tau(delta)
-        else:
-            logp = self._logp_chol(delta)
-
-        if onedim:
-            return logp[0]
-        return logp
-
-    def _logp_chol(self, delta):
-        chol_cov = self.chol_cov
-        _, k = delta.shape
-        k = pm.floatX(k)
-        diag = tt.nlinalg.diag(chol_cov)
-        # Check if the covariance matrix is positive definite.
-        ok = tt.all(diag > 0)
-        # If not, replace the diagonal. We return -inf later, but
-        # need to prevent solve_lower from throwing an exception.
-        chol_cov = tt.switch(ok, chol_cov, 1)
-
-        delta_trans = self.solve_lower(chol_cov, delta.T)
-
-        result = k * pm.floatX(np.log(2. * np.pi))
-        result += 2.0 * tt.sum(tt.log(diag))
-        result += (delta_trans ** 2).sum(axis=0)
-        result = -0.5 * result
-        return bound(result, ok)
-
-    def _logp_cov(self, delta):
-        chol_cov = self.chol_cov
-        _, k = delta.shape
-        k = pm.floatX(k)
-
-        diag = tt.nlinalg.diag(chol_cov)
-        ok = tt.all(diag > 0)
-
-        chol_cov = tt.switch(ok, chol_cov, 1)
-        diag = tt.nlinalg.diag(chol_cov)
-        delta_trans = self.solve_lower(chol_cov, delta.T)
-
-        result = k * pm.floatX(np.log(2. * np.pi))
-        result += 2.0 * tt.sum(tt.log(diag))
-        result += (delta_trans ** 2).sum(axis=0)
-        result = -0.5 * result
-        return bound(result, ok)
-
-    def _logp_tau(self, delta):
-        chol_tau = self.chol_tau
-        _, k = delta.shape
-        k = pm.floatX(k)
-
-        diag = tt.nlinalg.diag(chol_tau)
-        ok = tt.all(diag > 0)
-
-        chol_tau = tt.switch(ok, chol_tau, 1)
-        diag = tt.nlinalg.diag(chol_tau)
-        delta_trans = tt.dot(chol_tau.T, delta.T)
-
-        result = k * pm.floatX(np.log(2. * np.pi))
-        result -= 2.0 * tt.sum(tt.log(diag))
-        result += (delta_trans ** 2).sum(axis=0)
-        result = -0.5 * result
-        return bound(result, ok)
+        quaddist, logdet, ok = self._quaddist(value)
+        k = value.shape[-1].astype(theano.config.floatX)
+        norm = - 0.5 * k * pm.floatX(np.log(2 * np.pi))
+        return bound(norm - 0.5 * quaddist - logdet, ok)
 
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
             dist = self
         mu = dist.mu
-        try:
-            cov = dist.cov
-        except AttributeError:
-            cov = dist.chol_cov
         name_mu = get_variable_name(mu)
-        name_cov = get_variable_name(cov)
         return (r'${} \sim \text{{MvNormal}}'
-                r'(\mathit{{mu}}={}, \mathit{{cov}}={})$'
-                .format(name, name_mu, name_cov))
+                r'(\mathit{{mu}}={}, {})$'
+                .format(name, name_mu, self._repr_cov_params(dist)))
 
 
-class MvStudentT(Continuous):
+class MvStudentT(_QuadFormBase):
     R"""
     Multivariate Student-T log-likelihood.
 
@@ -298,7 +298,6 @@ class MvStudentT(Continuous):
                {\Sigma}^{-1}({\mathbf x}-{\mu})
              \right]^{(\nu+p)/2}}
 
-
     ========  =============================================
     Support   :math:`x \in \mathbb{R}^k`
     Mean      :math:`\mu` if :math:`\nu > 1` else undefined
@@ -306,65 +305,73 @@ class MvStudentT(Continuous):
                   if :math:`\nu>2` else undefined
     ========  =============================================
 
-
     Parameters
     ----------
     nu : int
         Degrees of freedom.
     Sigma : matrix
-        Covariance matrix.
+        Covariance matrix. Use `cov` in new code.
     mu : array
         Vector of means.
+    cov : matrix
+        The covariance matrix.
+    tau : matrix
+        The precision matrix.
+    chol : matrix
+        The cholesky factor of the covariance matrix.
+    lower : bool, default=True
+        Whether the cholesky fatcor is given as a lower triangular matrix.
     """
 
-    def __init__(self, nu, Sigma, mu=None, *args, **kwargs):
-        super(MvStudentT, self).__init__(*args, **kwargs)
+    def __init__(self, nu, Sigma=None, mu=None, cov=None, tau=None, chol=None,
+                 lower=None, *args, **kwargs):
+        if Sigma is not None:
+            if cov is not None:
+                raise ValueError('Specify only one of cov and Sigma')
+            cov = Sigma
+        super(MvStudentT, self).__init__(mu=mu, cov=cov, tau=tau, chol=chol,
+                                         lower=lower, *args, **kwargs)
         self.nu = nu = tt.as_tensor_variable(nu)
-        mu = tt.zeros(Sigma.shape[0]) if mu is None else tt.as_tensor_variable(mu)
-        self.Sigma = Sigma = tt.as_tensor_variable(Sigma)
-
-        self.mean = self.median = self.mode = self.mu = mu
+        self.mean = self.median = self.mode = self.mu = self.mu
 
     def random(self, point=None, size=None):
+        nu, mu = draw_values([self.nu, self.mu], point=point)
+        if self._cov_type == 'cov':
+            cov, = draw_values([self.cov], point=point)
+            dist = MvNormal.dist(mu=np.zeros_like(mu), cov=cov)
+        elif self._cov_type == 'tau':
+            tau, = draw_values([self.tau], point=point)
+            dist = MvNormal.dist(mu=np.zeros_like(mu), tau=tau)
+        else:
+            chol, = draw_values([self.chol_cov], point=point)
+            dist = MvNormal.dist(mu=np.zeros_like(mu), chol=chol)
+
+        samples = dist.random(point, size)
+
         chi2 = np.random.chisquare
-        mvn = np.random.multivariate_normal
-
-        nu, S, mu = draw_values([self.nu, self.Sigma, self.mu], point=point)
-
-        return (np.sqrt(nu) * (mvn(np.zeros(len(S)), S, size).T
-                               / chi2(nu, size))).T + mu
+        return (np.sqrt(nu) * samples.T / chi2(nu, size)).T + mu
 
     def logp(self, value):
+        quaddist, logdet, ok = self._quaddist(value)
+        k = value.shape[-1].astype(theano.config.floatX)
 
-        S = self.Sigma
-        nu = self.nu
-        mu = self.mu
-
-        d = S.shape[0]
-
-        X = value - mu
-
-        Q = X.dot(matrix_inverse(S)).dot(X.T).sum()
-        log_det = tt.log(det(S))
-        log_pdf = gammaln((nu + d) / 2.) - 0.5 * \
-            (d * tt.log(np.pi * nu) + log_det) - gammaln(nu / 2.)
-        log_pdf -= 0.5 * (nu + d) * tt.log(1 + Q / nu)
-
-        return log_pdf
+        norm = (gammaln((self.nu + k) / 2.)
+                - gammaln(self.nu / 2.)
+                - 0.5 * k * floatX(np.log(self.nu * np.pi)))
+        inner = - (self.nu + k) / 2. * tt.log1p(quaddist / self.nu)
+        return bound(norm + inner - logdet, ok)
 
     def _repr_latex_(self, name=None, dist=None):
         if dist is None:
             dist = self
         mu = dist.mu
         nu = dist.nu
-        Sigma = dist.Sigma
         name_nu = get_variable_name(nu)
         name_mu = get_variable_name(mu)
-        name_sigma = get_variable_name(Sigma)
         return (r'${} \sim \text{{MvStudentT}}'
                 r'(\mathit{{nu}}={}, \mathit{{mu}}={}, '
-                r'\mathit{{Sigma}}={})$'
-                .format(name, name_nu, name_mu, name_sigma))
+                r'{})$'
+                .format(name, name_nu, name_mu, self._repr_cov_params(dist)))
 
 
 class Dirichlet(Continuous):
@@ -466,38 +473,73 @@ class Multinomial(Discrete):
     Parameters
     ----------
     n : int or array
-        Number of trials (n > 0).
+        Number of trials (n > 0). If n is an array its shape must be (N,) with
+        N = p.shape[0]
     p : one- or two-dimensional array
         Probability of each one of the different outcomes. Elements must
-        be non-negative and sum to 1 along the last axis. They will be automatically
-        rescaled otherwise.
+        be non-negative and sum to 1 along the last axis. They will be
+        automatically rescaled otherwise.
     """
 
     def __init__(self, n, p, *args, **kwargs):
         super(Multinomial, self).__init__(*args, **kwargs)
 
         p = p / tt.sum(p, axis=-1, keepdims=True)
+        n = np.squeeze(n) # works also if n is a tensor
 
-        if len(self.shape) == 2:
+        if len(self.shape) > 1:
+            m = self.shape[-2]
             try:
-                assert n.shape == (self.shape[0],)
-            except AttributeError:
-                # this occurs when n is a scalar Python int or float
-                n *= tt.ones(self.shape[0])
-
+                assert n.shape == (m,)
+            except (AttributeError, AssertionError):
+                n = n * tt.ones(m)
             self.n = tt.shape_padright(n)
-            self.p = p if p.ndim == 2 else tt.shape_padleft(p)
+            self.p = p if p.ndim > 1 else tt.shape_padleft(p)
+        elif n.ndim == 1:
+            self.n = tt.shape_padright(n)
+            self.p = p if p.ndim > 1 else tt.shape_padleft(p)
         else:
+            # n is a scalar, p is a 1d array
             self.n = tt.as_tensor_variable(n)
             self.p = tt.as_tensor_variable(p)
 
         self.mean = self.n * self.p
-        self.mode = tt.cast(tround(self.mean), 'int32')
+        mode = tt.cast(tt.round(self.mean), 'int32')
+        diff = self.n - tt.sum(mode, axis=-1, keepdims=True)
+        inc_bool_arr = tt.abs_(diff) > 0
+        mode = tt.inc_subtensor(mode[inc_bool_arr.nonzero()],
+                                diff[inc_bool_arr.nonzero()])
+        self.mode = mode
 
     def _random(self, n, p, size=None):
+        original_dtype = p.dtype
+        # Set float type to float64 for numpy. This change is related to numpy issue #8317 (https://github.com/numpy/numpy/issues/8317)
+        p = p.astype('float64')
+        # Now, re-normalize all of the values in float64 precision. This is done inside the conditionals
         if size == p.shape:
             size = None
-        return np.random.multinomial(n, p, size=size)
+        if (n.ndim == 0) and (p.ndim == 1):
+            p = p / p.sum()
+            randnum = np.random.multinomial(n, p.squeeze(), size=size)
+        elif (n.ndim == 0) and (p.ndim > 1):
+            p = p / p.sum(axis=1, keepdims=True)
+            randnum = np.asarray([
+                np.random.multinomial(n.squeeze(), pp, size=size)
+                for pp in p
+            ])
+        elif (n.ndim > 0) and (p.ndim == 1):
+            p = p / p.sum()
+            randnum = np.asarray([
+                np.random.multinomial(nn, p.squeeze(), size=size)
+                for nn in n
+            ])
+        else:
+            p = p / p.sum(axis=1, keepdims=True)
+            randnum = np.asarray([
+                np.random.multinomial(nn, pp, size=size)
+                for (nn, pp) in zip(n, p)
+            ])
+        return randnum.astype(original_dtype)
 
     def random(self, point=None, size=None):
         n, p = draw_values([self.n, self.p], point=point)
@@ -563,7 +605,7 @@ class PosDefMatrix(theano.Op):
         try:
             z[0] = np.array(posdef(x), dtype='int8')
         except Exception:
-            pm._log.exception('Failed to check if positive definite', x)
+            pm._log.exception('Failed to check if %s positive definite', x)
             raise
 
     def infer_shape(self, node, shapes):
@@ -629,10 +671,16 @@ class Wishart(Continuous):
         self.p = p = tt.as_tensor_variable(V.shape[0])
         self.V = V = tt.as_tensor_variable(V)
         self.mean = nu * V
-        self.mode = tt.switch(1 * (nu >= p + 1),
+        self.mode = tt.switch(tt.ge(nu, p + 1),
                               (nu - p - 1) * V,
                               np.nan)
 
+    def random(self, point=None, size=None):
+        nu, V = draw_values([self.nu, self.V], point=point)
+        size= 1 if size is None else size
+        return generate_samples(stats.wishart.rvs, np.asscalar(nu), V,
+                                    broadcast_shape=(size,))
+
     def logp(self, X):
         nu = self.nu
         p = self.p
@@ -771,7 +819,7 @@ class LKJCholeskyCov(Continuous):
     Parameters
     ----------
     n : int
-        Dimension of the covariance matrix (n > 0).
+        Dimension of the covariance matrix (n > 1).
     eta : float
         The shape parameter (eta > 0) of the LKJ distribution. eta = 1
         implies a uniform distribution of the correlation matrices;
@@ -941,7 +989,7 @@ class LKJCorr(Continuous):
     Parameters
     ----------
     n : int
-        Dimension of the covariance matrix (n > 0).
+        Dimension of the covariance matrix (n > 1).
     eta : float
         The shape parameter (eta > 0) of the LKJ distribution. eta = 1
         implies a uniform distribution of the correlation matrices;
@@ -985,20 +1033,48 @@ class LKJCorr(Continuous):
             raise ValueError('Invalid parameter: please use eta as the shape parameter and '
                              'n as the dimension parameter.')
 
-        n_elem = int(n * (n - 1) / 2)
-        self.mean = np.zeros(n_elem, dtype=theano.config.floatX)
+        shape = n * (n - 1) // 2
+        self.mean = floatX(np.zeros(shape))
 
         if transform == 'interval':
             transform = transforms.interval(-1, 1)
 
-        super(LKJCorr, self).__init__(shape=n_elem, transform=transform,
+        super(LKJCorr, self).__init__(shape=shape, transform=transform,
                                       *args, **kwargs)
         warnings.warn('Parameters in LKJCorr have been rename: shape parameter n -> eta '
                       'dimension parameter p -> n. Please double check your initialization.',
                       DeprecationWarning)
         self.tri_index = np.zeros([n, n], dtype='int32')
-        self.tri_index[np.triu_indices(n, k=1)] = np.arange(n_elem)
-        self.tri_index[np.triu_indices(n, k=1)[::-1]] = np.arange(n_elem)
+        self.tri_index[np.triu_indices(n, k=1)] = np.arange(shape)
+        self.tri_index[np.triu_indices(n, k=1)[::-1]] = np.arange(shape)
+
+    def _random(self, n, eta, size=None):
+        size = size if isinstance(size, tuple) else (size,)
+        # original implementation in R see:
+        # https://github.com/rmcelreath/rethinking/blob/master/R/distributions.r
+        beta = eta - 1 + n/2
+        r12 = 2 * stats.beta.rvs(a=beta, b=beta, size=size) - 1
+        P = np.eye(n)[:, :, np.newaxis] * np.ones(size)
+        P[0, 1] = r12
+        P[1, 1] = np.sqrt(1 - r12**2)
+        if n > 2:
+            for m in range(1, n-1):
+                beta -= 0.5
+                y = stats.beta.rvs(a=(m+1) / 2., b=beta, size=size)
+                z = stats.norm.rvs(loc=0, scale=1, size=(m+1, ) + size)
+                z = z / np.sqrt(np.einsum('ij,ij->j', z, z))
+                P[0:m+1, m+1] = np.sqrt(y) * z
+                P[m+1, m+1] = np.sqrt(1 - y)
+        Pt = np.transpose(P, (2, 0 ,1))
+        C = np.einsum('...ji,...jk->...ik', Pt, Pt)
+        return C.transpose((1, 2, 0))[np.triu_indices(n, k=1)].T
+
+    def random(self, point=None, size=None):
+        n, eta = draw_values([self.n, self.eta], point=point)
+        size= 1 if size is None else size
+        samples = generate_samples(self._random, n, eta,
+                                   broadcast_shape=(size,))
+        return samples
 
     def logp(self, x):
         n = self.n
