@@ -1,4 +1,6 @@
 import collections
+import functools
+import itertools
 import threading
 import six
 
@@ -15,7 +17,7 @@ from .memoize import memoize
 from .theanof import gradient, hessian, inputvars, generator
 from .vartypes import typefilter, discrete_types, continuous_types, isgenerator
 from .blocking import DictToArrayBijection, ArrayOrdering
-from .util import get_transformed_name
+from .util import get_transformed_name, escape_latex
 
 __all__ = [
     'Model', 'Factor', 'compilef', 'fn', 'fastfn', 'modelcontext',
@@ -150,6 +152,9 @@ class Factor(object):
     """Common functionality for objects with a log probability density
     associated with them.
     """
+    def __init__(self, *args, **kwargs):
+        super(Factor, self).__init__(*args, **kwargs)
+
     @property
     def logp(self):
         """Compiled log probability density function"""
@@ -167,6 +172,18 @@ class Factor(object):
         """Compiled log probability density hessian function"""
         return self.model.fn(hessian(self.logpt, vars))
 
+    @property
+    def logp_nojac(self):
+        return self.model.fn(self.logp_nojact)
+
+    def dlogp_nojac(self, vars=None):
+        """Compiled log density gradient function, without jacobian terms."""
+        return self.model.fn(gradient(self.logp_nojact, vars))
+
+    def d2logp_nojac(self, vars=None):
+        """Compiled log density hessian function, without jacobian terms."""
+        return self.model.fn(hessian(self.logp_nojact, vars))
+
     @property
     def fastlogp(self):
         """Compiled log probability density function"""
@@ -180,13 +197,39 @@ class Factor(object):
         """Compiled log probability density hessian function"""
         return self.model.fastfn(hessian(self.logpt, vars))
 
+    @property
+    def fastlogp_nojac(self):
+        return self.model.fastfn(self.logp_nojact)
+
+    def fastdlogp_nojac(self, vars=None):
+        """Compiled log density gradient function, without jacobian terms."""
+        return self.model.fastfn(gradient(self.logp_nojact, vars))
+
+    def fastd2logp_nojac(self, vars=None):
+        """Compiled log density hessian function, without jacobian terms."""
+        return self.model.fastfn(hessian(self.logp_nojact, vars))
+
     @property
     def logpt(self):
         """Theano scalar of log-probability of the model"""
         if getattr(self, 'total_size', None) is not None:
-            return tt.sum(self.logp_elemwiset) * self.scaling
+            logp = self.logp_sum_unscaledt * self.scaling
         else:
-            return tt.sum(self.logp_elemwiset)
+            logp = self.logp_sum_unscaledt
+        if self.name is not None:
+            logp.name = '__logp_%s' % self.name
+        return logp
+
+    @property
+    def logp_nojact(self):
+        """Theano scalar of log-probability, excluding jacobian terms."""
+        if getattr(self, 'total_size', None) is not None:
+            logp = tt.sum(self.logp_nojac_unscaledt) * self.scaling
+        else:
+            logp = tt.sum(self.logp_nojac_unscaledt)
+        if self.name is not None:
+            logp.name = '__logp_%s' % self.name
+        return logp
 
 
 class InitContextMeta(type):
@@ -277,6 +320,173 @@ class treedict(dict):
             return dict.__contains__(self, item)
 
 
+class ValueGradFunction(object):
+    """Create a theano function that computes a value and its gradient.
+
+    Parameters
+    ----------
+    cost : theano variable
+        The value that we compute with its gradient.
+    grad_vars : list of named theano variables or None
+        The arguments with respect to which the gradient is computed.
+    extra_vars : list of named theano variables or None
+        Other arguments of the function that are assumed constant. They
+        are stored in shared variables and can be set using
+        `set_extra_values`.
+    dtype : str, default=theano.config.floatX
+        The dtype of the arrays.
+    casting : {'no', 'equiv', 'save', 'same_kind', 'unsafe'}, default='no'
+        Casting rule for casting `grad_args` to the array dtype.
+        See `numpy.can_cast` for a description of the options.
+        Keep in mind that we cast the variables to the array *and*
+        back from the array dtype to the variable dtype.
+    kwargs
+        Extra arguments are passed on to `theano.function`.
+
+    Attributes
+    ----------
+    size : int
+        The number of elements in the parameter array.
+    profile : theano profiling object or None
+        The profiling object of the theano function that computes value and
+        gradient. This is None unless `profile=True` was set in the
+        kwargs.
+    """
+    def __init__(self, cost, grad_vars, extra_vars=None, dtype=None,
+                 casting='no', **kwargs):
+        if extra_vars is None:
+            extra_vars = []
+
+        names = [arg.name for arg in grad_vars + extra_vars]
+        if any(name is None for name in names):
+            raise ValueError('Arguments must be named.')
+        if len(set(names)) != len(names):
+            raise ValueError('Names of the arguments are not unique.')
+
+        if cost.ndim > 0:
+            raise ValueError('Cost must be a scalar.')
+
+        self._grad_vars = grad_vars
+        self._extra_vars = extra_vars
+        self._extra_var_names = set(var.name for var in extra_vars)
+        self._cost = cost
+        self._ordering = ArrayOrdering(grad_vars)
+        self.size = self._ordering.size
+        self._extra_are_set = False
+        if dtype is None:
+            dtype = theano.config.floatX
+        self.dtype = dtype
+        for var in self._grad_vars:
+            if not np.can_cast(var.dtype, self.dtype, casting):
+                raise TypeError('Invalid dtype for variable %s. Can not '
+                                'cast to %s with casting rule %s.'
+                                % (var.name, self.dtype, casting))
+            if not np.issubdtype(var.dtype, float):
+                raise TypeError('Invalid dtype for variable %s. Must be '
+                                'floating point but is %s.'
+                                % (var.name, var.dtype))
+
+        givens = []
+        self._extra_vars_shared = {}
+        for var in extra_vars:
+            shared = theano.shared(var.tag.test_value, var.name + '_shared__')
+            self._extra_vars_shared[var.name] = shared
+            givens.append((var, shared))
+
+        self._vars_joined, self._cost_joined = self._build_joined(
+            self._cost, grad_vars, self._ordering.vmap)
+
+        grad = tt.grad(self._cost_joined, self._vars_joined)
+        grad.name = '__grad'
+
+        inputs = [self._vars_joined]
+
+        self._theano_function = theano.function(
+            inputs, [self._cost_joined, grad], givens=givens, **kwargs)
+
+    def set_extra_values(self, extra_vars):
+        self._extra_are_set = True
+        for var in self._extra_vars:
+            self._extra_vars_shared[var.name].set_value(extra_vars[var.name])
+
+    def get_extra_values(self):
+        if not self._extra_are_set:
+            raise ValueError('Extra values are not set.')
+
+        return {var.name: self._extra_vars_shared[var.name].get_value()
+                for var in self._extra_vars}
+
+    def __call__(self, array, grad_out=None, extra_vars=None):
+        if extra_vars is not None:
+            self.set_extra_values(extra_vars)
+
+        if not self._extra_are_set:
+            raise ValueError('Extra values are not set.')
+
+        if array.shape != (self.size,):
+            raise ValueError('Invalid shape for array. Must be %s but is %s.'
+                             % ((self.size,), array.shape))
+
+        if grad_out is None:
+            out = np.empty_like(array)
+        else:
+            out = grad_out
+
+        logp, dlogp = self._theano_function(array)
+        if grad_out is None:
+            return logp, dlogp
+        else:
+            out[...] = dlogp
+            return logp
+
+    @property
+    def profile(self):
+        """Profiling information of the underlying theano function."""
+        return self._theano_function.profile
+
+    def dict_to_array(self, point):
+        """Convert a dictionary with values for grad_vars to an array."""
+        array = np.empty(self.size, dtype=self.dtype)
+        for varmap in self._ordering.vmap:
+            array[varmap.slc] = point[varmap.var].ravel().astype(self.dtype)
+        return array
+
+    def array_to_dict(self, array):
+        """Convert an array to a dictionary containing the grad_vars."""
+        if array.shape != (self.size,):
+            raise ValueError('Array should have shape (%s,) but has %s'
+                             % (self.size, array.shape))
+        if array.dtype != self.dtype:
+            raise ValueError('Array has invalid dtype. Should be %s but is %s'
+                             % (self._dtype, self.dtype))
+        point = {}
+        for varmap in self._ordering.vmap:
+            data = array[varmap.slc].reshape(varmap.shp)
+            point[varmap.var] = data.astype(varmap.dtyp)
+
+        return point
+
+    def array_to_full_dict(self, array):
+        """Convert an array to a dictionary with grad_vars and extra_vars."""
+        point = self.array_to_dict(array)
+        for name, var in self._extra_vars_shared.items():
+            point[name] = var.get_value()
+        return point
+
+    def _build_joined(self, cost, args, vmap):
+        args_joined = tt.vector('__args_joined')
+        args_joined.tag.test_value = np.zeros(self.size, dtype=self.dtype)
+
+        joined_slices = {}
+        for vmap in vmap:
+            sliced = args_joined[vmap.slc].reshape(vmap.shp)
+            sliced.name = vmap.var
+            joined_slices[vmap.var] = sliced
+
+        replace = {var: joined_slices[var.name] for var in args}
+        return args_joined, theano.clone(cost, replace=replace)
+
+
 class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
     """Encapsulates the variables and likelihood factors of a model.
 
@@ -419,7 +629,6 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
         return bij
 
     @property
-    @memoize
     def dict_to_array(self):
         return self.bijection.map
 
@@ -428,23 +637,51 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
         return sum(var.dsize for var in self.free_RVs)
 
     @property
-    @memoize
     def logp_array(self):
         return self.bijection.mapf(self.fastlogp)
 
     @property
-    @memoize
     def dlogp_array(self):
         vars = inputvars(self.cont_vars)
         return self.bijection.mapf(self.fastdlogp(vars))
 
+    def logp_dlogp_function(self, grad_vars=None, **kwargs):
+        if grad_vars is None:
+            grad_vars = list(typefilter(self.free_RVs, continuous_types))
+        else:
+            for var in grad_vars:
+                if var.dtype not in continuous_types:
+                    raise ValueError("Can only compute the gradient of "
+                                     "continuous types: %s" % var)
+        varnames = [var.name for var in grad_vars]
+        extra_vars = [var for var in self.free_RVs if var.name not in varnames]
+        return ValueGradFunction(self.logpt, grad_vars, extra_vars, **kwargs)
+
     @property
-    @memoize
     def logpt(self):
         """Theano scalar of log-probability of the model"""
         with self:
-            factors = [var.logpt for var in self.basic_RVs] + self.potentials
-            return tt.add(*map(tt.sum, factors))
+            factors = [var.logpt for var in self.basic_RVs]
+            logp_factors = tt.sum(factors)
+            logp_potentials = tt.sum([tt.sum(pot) for pot in self.potentials])
+            logp = logp_factors + logp_potentials
+            if self.name:
+                logp.name = '__logp_%s' % self.name
+            else:
+                logp.name = '__logp'
+            return logp
+
+    @property
+    def logp_nojact(self):
+        """Theano scalar of log-probability of the model"""
+        with self:
+            factors = [var.logp_nojact for var in self.basic_RVs] + self.potentials
+            logp = tt.sum([tt.sum(factor) for factor in factors])
+            if self.name:
+                logp.name = '__logp_nojac_%s' % self.name
+            else:
+                logp.name = '__logp_nojac'
+            return logp
 
     @property
     def varlogpt(self):
@@ -452,7 +689,7 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
            (excluding deterministic)."""
         with self:
             factors = [var.logpt for var in self.vars]
-            return tt.add(*map(tt.sum, factors))
+            return tt.sum(factors)
 
     @property
     def vars(self):
@@ -595,7 +832,6 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
             except KeyError:
                 raise e
 
-    @memoize
     def makefn(self, outs, mode=None, *args, **kwargs):
         """Compiles a Theano function which returns `outs` and takes the variable
         ancestors of `outs` as inputs.
@@ -676,7 +912,7 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
 
         return f.profile
 
-    def flatten(self, vars=None):
+    def flatten(self, vars=None, order=None, inputvar=None):
         """Flattens model's input and returns:
             FlatView with
             * input vector variable
@@ -687,6 +923,10 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
         ----------
         vars : list of variables or None
             if None, then all model.free_RVs are used for flattening input
+        order : ArrayOrdering
+            Optional, use predefined ordering
+        inputvar : tt.vector
+            Optional, use predefined inputvar
 
         Returns
         -------
@@ -694,15 +934,29 @@ class Model(six.with_metaclass(InitContextMeta, Context, Factor)):
         """
         if vars is None:
             vars = self.free_RVs
-        order = ArrayOrdering(vars)
-        inputvar = tt.vector('flat_view', dtype=theano.config.floatX)
-        inputvar.tag.test_value = flatten_list(vars).tag.test_value
+        if order is None:
+            order = ArrayOrdering(vars)
+        if inputvar is None:
+            inputvar = tt.vector('flat_view', dtype=theano.config.floatX)
+            if theano.config.compute_test_value != 'off':
+                if vars:
+                    inputvar.tag.test_value = flatten_list(vars).tag.test_value
+                else:
+                    inputvar.tag.test_value = np.asarray([], inputvar.dtype)
         replacements = {self.named_vars[name]: inputvar[slc].reshape(shape).astype(dtype)
                         for name, slc, shape, dtype in order.vmap}
         view = {vm.var: vm for vm in order.vmap}
         flat_view = FlatView(inputvar, replacements, view)
         return flat_view
 
+    def _repr_latex_(self, name=None, dist=None):
+        tex_vars = []
+        for rv in itertools.chain(self.unobserved_RVs, self.observed_RVs):
+            tex_vars.append(rv.__latex__())
+        return u'$${}$$'.format('\\\\'.join([tex.strip('$') for tex in tex_vars if tex is not None]))
+
+    __latex__ = _repr_latex_
+
 
 def fn(outs, mode=None, model=None, *args, **kwargs):
     """Compiles a Theano function which returns the values of `outs` and
@@ -867,6 +1121,10 @@ class FreeRV(Factor, TensorVariable):
             self.tag.test_value = np.ones(
                 distribution.shape, distribution.dtype) * distribution.default()
             self.logp_elemwiset = distribution.logp(self)
+            # The logp might need scaling in minibatches.
+            # This is done in `Factor`.
+            self.logp_sum_unscaledt = distribution.logp_sum(self)
+            self.logp_nojac_unscaledt = distribution.logp_nojac(self)
             self.total_size = total_size
             self.model = model
             self.scaling = _get_scaling(total_size, self.shape, self.ndim)
@@ -882,7 +1140,9 @@ class FreeRV(Factor, TensorVariable):
             name = self.name
         if dist is None:
             dist = self.distribution
-        return self.distribution._repr_latex_(name=name, dist=dist)
+        return self.distribution._repr_latex_(name=escape_latex(name), dist=dist)
+
+    __latex__ = _repr_latex_
 
     @property
     def init_value(self):
@@ -968,6 +1228,10 @@ class ObservedRV(Factor, TensorVariable):
 
             self.missing_values = data.missing_values
             self.logp_elemwiset = distribution.logp(data)
+            # The logp might need scaling in minibatches.
+            # This is done in `Factor`.
+            self.logp_sum_unscaledt = distribution.logp_sum(data)
+            self.logp_nojac_unscaledt = distribution.logp_nojac(data)
             self.total_size = total_size
             self.model = model
             self.distribution = distribution
@@ -985,7 +1249,9 @@ class ObservedRV(Factor, TensorVariable):
             name = self.name
         if dist is None:
             dist = self.distribution
-        return self.distribution._repr_latex_(name=name, dist=dist)
+        return self.distribution._repr_latex_(name=escape_latex(name), dist=dist)
+
+    __latex__ = _repr_latex_
 
     @property
     def init_value(self):
@@ -1017,12 +1283,36 @@ class MultiObservedRV(Factor):
         self.missing_values = [datum.missing_values for datum in self.data.values()
                                if datum.missing_values is not None]
         self.logp_elemwiset = distribution.logp(**self.data)
+        # The logp might need scaling in minibatches.
+        # This is done in `Factor`.
+        self.logp_sum_unscaledt = distribution.logp_sum(**self.data)
+        self.logp_nojac_unscaledt = distribution.logp_nojac(**self.data)
         self.total_size = total_size
         self.model = model
         self.distribution = distribution
         self.scaling = _get_scaling(total_size, self.logp_elemwiset.shape, self.logp_elemwiset.ndim)
 
 
+def _walk_up_rv(rv):
+    """Walk up theano graph to get inputs for deterministic RV."""
+    all_rvs = []
+    parents = list(itertools.chain(*[j.inputs for j in rv.get_parents()]))
+    if parents:
+        for parent in parents:
+            all_rvs.extend(_walk_up_rv(parent))
+    else:
+        if rv.name:
+            all_rvs.append(rv.name)
+        else:
+            all_rvs.append(r'\text{Constant}')
+    return all_rvs
+
+
+def _latex_repr_rv(rv):
+    """Make latex string for a Deterministic variable"""
+    return r'${} \sim \text{{Deterministic}}({})$'.format(rv.name, r', '.join(_walk_up_rv(rv)))
+
+
 def Deterministic(name, var, model=None):
     """Create a named deterministic variable
 
@@ -1039,6 +1329,8 @@ def Deterministic(name, var, model=None):
     var.name = model.name_for(name)
     model.deterministics.append(var)
     model.add_random_variable(var)
+    var._repr_latex_ = functools.partial(_latex_repr_rv, var)
+    var.__latex__ = var._repr_latex_
     return var
 
 
@@ -1110,7 +1402,9 @@ class TransformedRV(TensorVariable):
             name = self.name
         if dist is None:
             dist = self.distribution
-        return self.distribution._repr_latex_(name=name, dist=dist)
+        return self.distribution._repr_latex_(name=escape_latex(name), dist=dist)
+
+    __latex__ = _repr_latex_
 
     @property
     def init_value(self):
