@@ -1,4 +1,4 @@
-from collections import defaultdict
+from collections import defaultdict, Sequence
 
 from joblib import Parallel, delayed
 from numpy.random import randint, seed
@@ -8,19 +8,19 @@ import pymc3 as pm
 from .backends.base import merge_traces, BaseTrace, MultiTrace
 from .backends.ndarray import NDArray
 from .model import modelcontext, Point
-from .step_methods import (NUTS, HamiltonianMC, Metropolis, BinaryMetropolis,
+from .step_methods import (NUTS, HamiltonianMC, SGFS, Metropolis, BinaryMetropolis,
                            BinaryGibbsMetropolis, CategoricalGibbsMetropolis,
                            Slice, CompoundStep)
-from .plots.traceplot import traceplot
-from .util import is_transformed_name, get_untransformed_name
+from .util import update_start_vals
+from pymc3.step_methods.hmc import quadpotential
 from tqdm import tqdm
 
 import sys
 sys.setrecursionlimit(10000)
 
-__all__ = ['sample', 'iter_sample', 'sample_ppc', 'init_nuts']
+__all__ = ['sample', 'iter_sample', 'sample_ppc', 'sample_ppc_w', 'init_nuts']
 
-STEP_METHODS = (NUTS, HamiltonianMC, Metropolis, BinaryMetropolis,
+STEP_METHODS = (NUTS, HamiltonianMC, SGFS, Metropolis, BinaryMetropolis,
                 BinaryGibbsMetropolis, Slice, CategoricalGibbsMetropolis)
 
 
@@ -118,24 +118,35 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None,
         A step function or collection of functions. If there are variables
         without a step methods, step methods for those variables will
         be assigned automatically.
-    init : str {'ADVI', 'ADVI_MAP', 'MAP', 'NUTS', 'auto', None}
-        Initialization method to use. Only works for auto-assigned step methods.
+    init : str
+        Initialization method to use for auto-assigned NUTS samplers.
 
-        * ADVI: Run ADVI to estimate starting points and diagonal covariance
-          matrix. If njobs > 1 it will sample starting points from the estimated
-          posterior, otherwise it will use the estimated posterior mean.
-        * ADVI_MAP: Initialize ADVI with MAP and use MAP as starting point.
-        * MAP: Use the MAP as starting point.
-        * NUTS: Run NUTS to estimate starting points and covariance matrix. If
-          njobs > 1 it will sample starting points from the estimated posterior,
-          otherwise it will use the estimated posterior mean.
-        * auto : Auto-initialize, if possible. Currently only works when NUTS
-          is auto-assigned as step method (default).
-        * None: Do not initialize.
+        * auto : Choose a default initialization method automatically.
+          Currently, this is `'jitter+adapt_diag'`, but this can change in
+          the future. If you depend on the exact behaviour, choose an
+          initialization method explicitly.
+        * adapt_diag : Start with a identity mass matrix and then adapt
+          a diagonal based on the variance of the tuning samples. All
+          chains use the test value (usually the prior mean) as starting
+          point.
+        * jitter+adapt_diag : Same as `adapt_diag`, but add uniform jitter
+          in [-1, 1] to the starting point in each chain.
+        * advi+adapt_diag : Run ADVI and then adapt the resulting diagonal
+          mass matrix based on the sample variance of the tuning samples.
+        * advi+adapt_diag_grad : Run ADVI and then adapt the resulting
+          diagonal mass matrix based on the variance of the gradients
+          during tuning. This is **experimental** and might be removed
+          in a future release.
+        * advi : Run ADVI to estimate posterior mean and diagonal mass
+          matrix.
+        * advi_map: Initialize ADVI with MAP and use MAP as starting point.
+        * map : Use the MAP as starting point. This is discouraged.
+        * nuts : Run NUTS and estimate posterior mean and mass matrix from
+          the trace.
     n_init : int
         Number of iterations of initializer
         If 'ADVI', number of iterations, if 'nuts', number of draws.
-    start : dict
+    start : dict, or array of dict
         Starting point in parameter space (or partial point)
         Defaults to trace.point(-1)) if there is a trace provided and
         model.test_point if not (defaults to empty dict).
@@ -218,10 +229,10 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None,
     """
     model = modelcontext(model)
 
-    draws += tune
+    if start is not None:
+        _check_start_shape(model, start)
 
-    if init is not None:
-        init = init.lower()
+    draws += tune
 
     if nuts_kwargs is not None:
         if step_kwargs is not None:
@@ -236,8 +247,6 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None,
         pm._log.info('Auto-assigning NUTS sampler...')
         args = step_kwargs if step_kwargs is not None else {}
         args = args.get('nuts', {})
-        if init == 'auto':
-            init = 'ADVI'
         start_, step = init_nuts(init=init, njobs=njobs, n_init=n_init,
                                  model=model, random_seed=random_seed,
                                  progressbar=progressbar, **args)
@@ -276,6 +285,38 @@ def sample(draws=500, step=None, init='auto', n_init=200000, start=None,
     return sample_func(**sample_args)[discard:]
 
 
+def _check_start_shape(model, start):
+    e = ''
+    if isinstance(start, (Sequence, np.ndarray)):
+        # to deal with iterable start argument
+        for start_iter in start:
+            _check_start_shape(model, start_iter)
+        return
+    elif not isinstance(start, dict):
+        raise TypeError("start argument must be a dict "
+                        "or an array-like of dicts")
+    for var in model.vars:
+        if var.name in start.keys():
+            var_shape = var.shape.tag.test_value
+            start_var_shape = np.shape(start[var.name])
+            if start_var_shape:
+                if not np.array_equal(var_shape, start_var_shape):
+                    e += "\nExpected shape {} for var '{}', got: {}".format(
+                        tuple(var_shape), var.name, start_var_shape
+                    )
+            # if start var has no shape
+            else:
+                # if model var has a specified shape
+                if var_shape:
+                    e += "\nExpected shape {} for var " \
+                         "'{}', got scalar {}".format(
+                             tuple(var_shape), var.name, start[var.name]
+                         )
+
+    if e != '':
+        raise ValueError("Bad shape for start argument:{}".format(e))
+
+
 def _sample(draws, step=None, start=None, trace=None, chain=0, tune=None,
             progressbar=True, model=None, random_seed=-1, live_plot=False,
             live_plot_kwargs=None, **kwargs):
@@ -295,9 +336,9 @@ def _sample(draws, step=None, start=None, trace=None, chain=0, tune=None,
                 if it >= skip_first:
                     trace = MultiTrace([strace])
                     if it == skip_first:
-                        ax = traceplot(trace, live_plot=False, **live_plot_kwargs)
+                        ax = pm.plots.traceplot(trace, live_plot=False, **live_plot_kwargs)
                     elif (it - skip_first) % refresh_every == 0 or it == draws - 1:
-                        traceplot(trace, ax=ax, live_plot=True, **live_plot_kwargs)
+                        pm.plots.traceplot(trace, ax=ax, live_plot=True, **live_plot_kwargs)
     except KeyboardInterrupt:
         pass
     finally:
@@ -365,9 +406,9 @@ def _iter_sample(draws, step, start=None, trace=None, chain=0, tune=None,
     strace = _choose_backend(trace, chain, model=model)
 
     if len(strace) > 0:
-        _update_start_vals(start, strace.point(-1), model)
+        update_start_vals(start, strace.point(-1), model)
     else:
-        _update_start_vals(start, model.test_point, model)
+        update_start_vals(start, model.test_point, model)
 
     try:
         step = CompoundStep(step)
@@ -474,20 +515,6 @@ def stop_tuning(step):
 
     return step
 
-def _update_start_vals(a, b, model):
-    """Update a with b, without overwriting existing keys. Values specified for
-    transformed variables on the original scale are also transformed and inserted.
-    """
-    if model is not None:
-        for free_RV in model.free_RVs:
-            tname = free_RV.name
-            for name in a:
-                if is_transformed_name(tname) and get_untransformed_name(tname) == name:
-                    transform_func = [d.transformation for d in model.deterministics if d.name == name]
-                    if transform_func:
-                        b[tname] = transform_func[0].forward_val(a[name], point=b).eval()
-
-    a.update({k: v for k, v in b.items() if k not in a})
 
 def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
                random_seed=None, progressbar=True):
@@ -496,7 +523,7 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
     Parameters
     ----------
     trace : backend, list, or MultiTrace
-        Trace generated from MCMC sampling
+        Trace generated from MCMC sampling.
     samples : int
         Number of posterior predictive samples to generate. Defaults to the
         length of `trace`
@@ -508,36 +535,45 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
     size : int
         The number of random draws from the distribution specified by the
         parameters in each sample of the trace.
+    random_seed : int
+        Seed for the random number generator.
+    progressbar : bool
+        Whether or not to display a progress bar in the command line. The
+        bar shows the percentage of completion, the sampling speed in
+        samples per second (SPS), and the estimated remaining time until
+        completion ("expected time of arrival"; ETA).
 
     Returns
     -------
     samples : dict
-        Dictionary with the variables as keys. The values corresponding
-        to the posterior predictive samples.
+        Dictionary with the variables as keys. The values corresponding to the
+        posterior predictive samples.
     """
     if samples is None:
         samples = len(trace)
 
-    if model is None:
-        model = modelcontext(model)
+    model = modelcontext(model)
 
     if vars is None:
         vars = model.observed_RVs
 
     seed(random_seed)
 
+    indices = randint(0, len(trace), samples)
     if progressbar:
-        indices = tqdm(randint(0, len(trace), samples), total=samples)
-    else:
-        indices = randint(0, len(trace), samples)
+        indices = tqdm(indices, total=samples)
 
     try:
         ppc = defaultdict(list)
         for idx in indices:
             param = trace[idx]
             for var in vars:
-                vals = var.distribution.random(point=param, size=size)
-                ppc[var.name].append(vals)
+                ppc[var.name].append(var.distribution.random(point=param,
+                                                             size=size))
+
+    except KeyboardInterrupt:
+        pass
+
     finally:
         if progressbar:
             indices.close()
@@ -545,28 +581,149 @@ def sample_ppc(trace, samples=None, model=None, vars=None, size=None,
     return {k: np.asarray(v) for k, v in ppc.items()}
 
 
-def init_nuts(init='ADVI', njobs=1, n_init=500000, model=None,
-              random_seed=-1, progressbar=True, **kwargs):
-    """Initialize and sample from posterior of a continuous model.
-
-    This is a convenience function. NUTS convergence and sampling speed is extremely
-    dependent on the choice of mass/scaling matrix. In our experience, using ADVI
-    to estimate a diagonal covariance matrix and using this as the scaling matrix
-    produces robust results over a wide class of continuous models.
+def sample_ppc_w(traces, samples=None, models=None, size=None, weights=None,
+                 random_seed=None, progressbar=True):
+    """Generate weighted posterior predictive samples from a list of models and
+    a list of traces according to a set of weights.
 
     Parameters
     ----------
-    init : str {'ADVI', 'ADVI_MAP', 'MAP', 'NUTS'}
+    traces : list
+        List of traces generated from MCMC sampling. The number of traces should
+        be equal to the number of weights.
+    samples : int
+        Number of posterior predictive samples to generate. Defaults to the
+        length of the shorter trace in traces.
+    models : list
+        List of models used to generate the list of traces. The number of models
+        should be equal to the number of weights and the number of observed RVs
+        should be the same for all models.
+        By default a single model will be inferred from `with` context, in this
+        case results will only be meaningful if all models share the same
+        distributions for the observed RVs.
+    size : int
+        The number of random draws from the distributions specified by the
+        parameters in each sample of the trace.
+    weights: array-like
+        Individual weights for each trace. Default, same weight for each model.
+    random_seed : int
+        Seed for the random number generator.
+    progressbar : bool
+        Whether or not to display a progress bar in the command line. The
+        bar shows the percentage of completion, the sampling speed in
+        samples per second (SPS), and the estimated remaining time until
+        completion ("expected time of arrival"; ETA).
+
+    Returns
+    -------
+    samples : dict
+        Dictionary with the variables as keys. The values corresponding to the
+        posterior predictive samples from the weighted models.
+    """
+    seed(random_seed)
+
+    if models is None:
+        models = [modelcontext(models)] * len(traces)
+
+    if weights is None:
+        weights = [1] * len(traces)
+
+    if len(traces) != len(weights):
+        raise ValueError('The number of traces and weights should be the same')
+
+    if len(models) != len(weights):
+        raise ValueError('The number of models and weights should be the same')
+
+    lenght_morv = len(models[0].observed_RVs)
+    if not all(len(i.observed_RVs) == lenght_morv for i in models):
+        raise ValueError(
+            'The number of observed RVs should be the same for all models')
+
+    weights = np.asarray(weights)
+    p = weights / np.sum(weights)
+
+    min_tr = min([len(i) for i in traces])
+
+    n = (min_tr * p).astype('int')
+    # ensure n sum up to min_tr
+    idx = np.argmax(n)
+    n[idx] = n[idx] + min_tr - np.sum(n)
+
+    trace = np.concatenate([np.random.choice(traces[i], j)
+                            for i, j in enumerate(n)])
+
+    variables = []
+    for i, m in enumerate(models):
+        variables.extend(m.observed_RVs * n[i])
+
+    len_trace = len(trace)
+
+    if samples is None:
+        samples = len_trace
+
+    indices = randint(0, len_trace, samples)
+
+    if progressbar:
+        indices = tqdm(indices, total=samples)
+
+    try:
+        ppc = defaultdict(list)
+        for idx in indices:
+            param = trace[idx]
+            var = variables[idx]
+            ppc[var.name].append(var.distribution.random(point=param,
+                                                         size=size))
+
+    except KeyboardInterrupt:
+        pass
+
+    finally:
+        if progressbar:
+            indices.close()
+
+    return {k: np.asarray(v) for k, v in ppc.items()}
+
+
+def init_nuts(init='auto', njobs=1, n_init=500000, model=None,
+              random_seed=-1, progressbar=True, **kwargs):
+    """Set up the mass matrix initialization for NUTS.
+
+    NUTS convergence and sampling speed is extremely dependent on the
+    choice of mass/scaling matrix. This function implements different
+    methods for choosing or adapting the mass matrix.
+
+    Parameters
+    ----------
+    init : str
         Initialization method to use.
-        * ADVI : Run ADVI to estimate posterior mean and diagonal covariance matrix.
-        * ADVI_MAP: Initialize ADVI with MAP and use MAP as starting point.
-        * MAP : Use the MAP as starting point.
-        * NUTS : Run NUTS and estimate posterior mean and covariance matrix.
+
+        * auto : Choose a default initialization method automatically.
+          Currently, this is `'jitter+adapt_diag'`, but this can change in
+          the future. If you depend on the exact behaviour, choose an
+          initialization method explicitly.
+        * adapt_diag : Start with a identity mass matrix and then adapt
+          a diagonal based on the variance of the tuning samples. All
+          chains use the test value (usually the prior mean) as starting
+          point.
+        * jitter+adapt_diag : Same as `adapt_diag`, but add uniform jitter
+          in [-1, 1] to the starting point in each chain.
+        * advi+adapt_diag : Run ADVI and then adapt the resulting diagonal
+          mass matrix based on the sample variance of the tuning samples.
+        * advi+adapt_diag_grad : Run ADVI and then adapt the resulting
+          diagonal mass matrix based on the variance of the gradients
+          during tuning. This is **experimental** and might be removed
+          in a future release.
+        * advi : Run ADVI to estimate posterior mean and diagonal mass
+          matrix.
+        * advi_map: Initialize ADVI with MAP and use MAP as starting point.
+        * map : Use the MAP as starting point. This is discouraged.
+        * nuts : Run NUTS and estimate posterior mean and mass matrix from
+          the trace.
     njobs : int
         Number of parallel jobs to start.
     n_init : int
         Number of iterations of initializer
-        If 'ADVI', number of iterations, if 'metropolis', number of draws.
+        If 'ADVI', number of iterations, if 'nuts', number of draws.
     model : Model (optional if in `with` context)
     progressbar : bool
         Whether or not to display a progressbar for advi sampling.
@@ -580,20 +737,95 @@ def init_nuts(init='ADVI', njobs=1, n_init=500000, model=None,
     nuts_sampler : pymc3.step_methods.NUTS
         Instantiated and initialized NUTS sampler object
     """
-
     model = pm.modelcontext(model)
 
+    vars = kwargs.get('vars', model.vars)
+    if set(vars) != set(model.vars):
+        raise ValueError('Must use init_nuts on all variables of a model.')
+    if not pm.model.all_continuous(vars):
+        raise ValueError('init_nuts can only be used for models with only '
+                         'continuous variables.')
+
+    if not isinstance(init, str):
+        raise TypeError('init must be a string.')
+
+    if init is not None:
+        init = init.lower()
+
+    if init == 'auto':
+        init = 'jitter+adapt_diag'
+
     pm._log.info('Initializing NUTS using {}...'.format(init))
 
     random_seed = int(np.atleast_1d(random_seed)[0])
 
-    if init is not None:
-        init = init.lower()
     cb = [
-        pm.callbacks.CheckParametersConvergence(tolerance=1e-2, diff='absolute'),
-        pm.callbacks.CheckParametersConvergence(tolerance=1e-2, diff='relative'),
+        pm.callbacks.CheckParametersConvergence(
+            tolerance=1e-2, diff='absolute'),
+        pm.callbacks.CheckParametersConvergence(
+            tolerance=1e-2, diff='relative'),
     ]
-    if init == 'advi':
+
+    if init == 'adapt_diag':
+        start = [model.test_point] * njobs
+        mean = np.mean([model.dict_to_array(vals) for vals in start], axis=0)
+        var = np.ones_like(mean)
+        potential = quadpotential.QuadPotentialDiagAdapt(
+            model.ndim, mean, var, 10)
+        if njobs == 1:
+            start = start[0]
+    elif init == 'jitter+adapt_diag':
+        start = []
+        for _ in range(njobs):
+            mean = {var: val.copy() for var, val in model.test_point.items()}
+            for val in mean.values():
+                val[...] += 2 * np.random.rand(*val.shape) - 1
+            start.append(mean)
+        mean = np.mean([model.dict_to_array(vals) for vals in start], axis=0)
+        var = np.ones_like(mean)
+        potential = quadpotential.QuadPotentialDiagAdapt(
+            model.ndim, mean, var, 10)
+        if njobs == 1:
+            start = start[0]
+    elif init == 'advi+adapt_diag_grad':
+        approx = pm.fit(
+            random_seed=random_seed,
+            n=n_init, method='advi', model=model,
+            callbacks=cb,
+            progressbar=progressbar,
+            obj_optimizer=pm.adagrad_window,
+        )  # type: pm.MeanField
+        start = approx.sample(draws=njobs)
+        start = list(start)
+        stds = approx.bij.rmap(approx.std.eval())
+        cov = model.dict_to_array(stds) ** 2
+        mean = approx.bij.rmap(approx.mean.get_value())
+        mean = model.dict_to_array(mean)
+        weight = 50
+        potential = quadpotential.QuadPotentialDiagAdaptGrad(
+            model.ndim, mean, cov, weight)
+        if njobs == 1:
+            start = start[0]
+    elif init == 'advi+adapt_diag':
+        approx = pm.fit(
+            random_seed=random_seed,
+            n=n_init, method='advi', model=model,
+            callbacks=cb,
+            progressbar=progressbar,
+            obj_optimizer=pm.adagrad_window,
+        )  # type: pm.MeanField
+        start = approx.sample(draws=njobs)
+        start = list(start)
+        stds = approx.bij.rmap(approx.std.eval())
+        cov = model.dict_to_array(stds) ** 2
+        mean = approx.bij.rmap(approx.mean.get_value())
+        mean = model.dict_to_array(mean)
+        weight = 50
+        potential = quadpotential.QuadPotentialDiagAdapt(
+            model.ndim, mean, cov, weight)
+        if njobs == 1:
+            start = start[0]
+    elif init == 'advi':
         approx = pm.fit(
             random_seed=random_seed,
             n=n_init, method='advi', model=model,
@@ -602,39 +834,48 @@ def init_nuts(init='ADVI', njobs=1, n_init=500000, model=None,
             obj_optimizer=pm.adagrad_window
         )  # type: pm.MeanField
         start = approx.sample(draws=njobs)
-        stds = approx.gbij.rmap(approx.std.eval())
+        start = list(start)
+        stds = approx.bij.rmap(approx.std.eval())
         cov = model.dict_to_array(stds) ** 2
+        potential = quadpotential.QuadPotentialDiag(cov)
         if njobs == 1:
             start = start[0]
     elif init == 'advi_map':
-        start = pm.find_MAP()
+        start = pm.find_MAP(include_transformed=True)
         approx = pm.MeanField(model=model, start=start)
         pm.fit(
             random_seed=random_seed,
-            n=n_init, method=pm.ADVI.from_mean_field(approx),
+            n=n_init, method=pm.KLqp(approx),
             callbacks=cb,
             progressbar=progressbar,
             obj_optimizer=pm.adagrad_window
         )
         start = approx.sample(draws=njobs)
-        stds = approx.gbij.rmap(approx.std.eval())
+        start = list(start)
+        stds = approx.bij.rmap(approx.std.eval())
         cov = model.dict_to_array(stds) ** 2
+        potential = quadpotential.QuadPotentialDiag(cov)
         if njobs == 1:
             start = start[0]
     elif init == 'map':
-        start = pm.find_MAP()
+        start = pm.find_MAP(include_transformed=True)
         cov = pm.find_hessian(point=start)
+        start = [start] * njobs
+        potential = quadpotential.QuadPotentialFull(cov)
+        if njobs == 1:
+            start = start[0]
     elif init == 'nuts':
         init_trace = pm.sample(draws=n_init, step=pm.NUTS(),
                                tune=n_init // 2,
                                random_seed=random_seed)
         cov = np.atleast_1d(pm.trace_cov(init_trace))
-        start = np.random.choice(init_trace, njobs)
+        start = list(np.random.choice(init_trace, njobs))
+        potential = quadpotential.QuadPotentialFull(cov)
         if njobs == 1:
             start = start[0]
     else:
         raise NotImplementedError('Initializer {} is not supported.'.format(init))
 
-    step = pm.NUTS(scaling=cov, is_cov=True, **kwargs)
+    step = pm.NUTS(potential=potential, **kwargs)
 
     return start, step
