@@ -1,17 +1,17 @@
 from collections import namedtuple
 import warnings
 
+import numpy as np
+import numpy.random as nr
+from scipy import stats, linalg
+import six
+
 from ..arraystep import Competence
 from pymc3.exceptions import SamplingError
 from .base_hmc import BaseHMC
 from pymc3.theanof import floatX
 from pymc3.vartypes import continuous_types
 
-import numpy as np
-import numpy.random as nr
-from scipy import stats, linalg
-import six
-
 __all__ = ['NUTS']
 
 
@@ -28,7 +28,7 @@ class NUTS(BaseHMC):
     sample. A detailed description can be found at [1], "Algorithm 6:
     Efficient No-U-Turn Sampler with Dual Averaging".
 
-    Nuts provides a number of statistics that can be accessed with
+    NUTS provides a number of statistics that can be accessed with
     `trace.get_sampler_stats`:
 
     - `mean_tree_accept`: The mean acceptance probability for the tree
@@ -70,6 +70,7 @@ class NUTS(BaseHMC):
     .. [1] Hoffman, Matthew D., & Gelman, Andrew. (2011). The No-U-Turn Sampler:
        Adaptively Setting Path Lengths in Hamiltonian Monte Carlo.
     """
+
     name = 'nuts'
 
     default_blocked = True
@@ -89,8 +90,11 @@ class NUTS(BaseHMC):
 
     def __init__(self, vars=None, Emax=1000, target_accept=0.8,
                  gamma=0.05, k=0.75, t0=10, adapt_step_size=True,
-                 max_treedepth=10, on_error='summary', **kwargs):
-        R"""
+                 max_treedepth=10, on_error='summary',
+                 early_max_treedepth=8,
+                 **kwargs):
+        R"""Set up the No-U-Turn sampler.
+
         Parameters
         ----------
         vars : list of Theano variables, default all continuous vars
@@ -116,6 +120,8 @@ class NUTS(BaseHMC):
         max_treedepth : int, default=10
             The maximum tree depth. Trajectories are stoped when this
             depth is reached.
+        early_max_treedepth : int, default=8
+            The maximum tree depth during the first 200 tuning samples.
         integrator : str, default "leapfrog"
             The integrator to use for the trajectories. One of "leapfrog",
             "two-stage" or "three-stage". The second two can increase
@@ -146,8 +152,7 @@ class NUTS(BaseHMC):
         This is usually achieved by setting the `tune` parameter if
         `pm.sample` to the desired number of tuning steps.
         """
-        super(NUTS, self).__init__(vars, use_single_leapfrog=True, **kwargs)
-
+        super(NUTS, self).__init__(vars, **kwargs)
         self.Emax = Emax
 
         self.target_accept = target_accept
@@ -162,17 +167,19 @@ class NUTS(BaseHMC):
         self.m = 1
         self.adapt_step_size = adapt_step_size
         self.max_treedepth = max_treedepth
+        self.early_max_treedepth = early_max_treedepth
 
         self.tune = True
         self.report = NutsReport(on_error, max_treedepth, target_accept)
 
     def astep(self, q0):
+        """Perform a single NUTS iteration."""
         p0 = self.potential.random()
-        v0 = self.compute_velocity(p0)
-        start_energy = self.compute_energy(q0, p0)
-        if not np.isfinite(start_energy):
+        start = self.integrator.compute_state(q0, p0)
+
+        if not np.isfinite(start.energy):
             raise ValueError('Bad initial energy: %s. The model '
-                             'might be misspecified.' % start_energy)
+                             'might be misspecified.' % start.energy)
 
         if not self.adapt_step_size:
             step_size = self.step_size
@@ -181,17 +188,21 @@ class NUTS(BaseHMC):
         else:
             step_size = np.exp(self.log_step_size_bar)
 
-        start = Edge(q0, p0, v0, self.dlogp(q0), start_energy)
-        tree = _Tree(len(p0), self.leapfrog, start, step_size, self.Emax)
+        if self.tune and self.m < 200:
+            max_treedepth = self.early_max_treedepth
+        else:
+            max_treedepth = self.max_treedepth
 
-        for _ in range(self.max_treedepth):
+        tree = _Tree(len(p0), self.integrator, start, step_size, self.Emax)
+
+        for _ in range(max_treedepth):
             direction = logbern(np.log(0.5)) * 2 - 1
-            diverging, turning = tree.extend(direction)
-            q = tree.proposal.q
+            diverging_info, turning = tree.extend(direction)
+            q, q_grad = tree.proposal.q, tree.proposal.q_grad
 
-            if diverging or turning:
-                if diverging:
-                    self.report._add_divergence(self.tune, *diverging)
+            if diverging_info or turning:
+                if diverging_info:
+                    self.report._add_divergence(self.tune, *diverging_info)
                 break
 
         w = 1. / (self.m + self.t0)
@@ -205,11 +216,14 @@ class NUTS(BaseHMC):
 
         self.m += 1
 
+        if self.tune:
+            self.potential.adapt(q, q_grad)
+
         stats = {
             'step_size': step_size,
             'tune': self.tune,
             'step_size_bar': np.exp(self.log_step_size_bar),
-            'diverging': diverging,
+            'diverging': bool(diverging_info),
         }
 
         stats.update(tree.stats())
@@ -218,16 +232,14 @@ class NUTS(BaseHMC):
 
     @staticmethod
     def competence(var):
+        """Check how appropriate this class is for sampling a random variable."""
         if var.dtype in continuous_types:
             return Competence.IDEAL
         return Competence.INCOMPATIBLE
 
 
-# A node in the NUTS tree that is at the far right or left of the tree
-Edge = namedtuple("Edge", 'q, p, v, q_grad, energy')
-
 # A proposal for the next position
-Proposal = namedtuple("Proposal", "q, energy, p_accept")
+Proposal = namedtuple("Proposal", "q, q_grad, energy, p_accept")
 
 # A subtree of the binary tree built by nuts.
 Subtree = namedtuple(
@@ -236,14 +248,14 @@ Subtree = namedtuple(
 
 
 class _Tree(object):
-    def __init__(self, ndim, leapfrog, start, step_size, Emax):
+    def __init__(self, ndim, integrator, start, step_size, Emax):
         """Binary tree from the NUTS algorithm.
 
         Parameters
         ----------
         leapfrog : function
             A function that performs a single leapfrog step.
-        start : Edge
+        start : integration.State
             The starting point of the trajectory.
         step_size : float
             The step size to use in this tree
@@ -252,14 +264,14 @@ class _Tree(object):
             transition as diverging.
         """
         self.ndim = ndim
-        self.leapfrog = leapfrog
+        self.integrator = integrator
         self.start = start
         self.step_size = step_size
         self.Emax = Emax
         self.start_energy = np.array(start.energy)
 
         self.left = self.right = start
-        self.proposal = Proposal(start.q, start.energy, 1.0)
+        self.proposal = Proposal(start.q, start.q_grad, start.energy, 1.0)
         self.depth = 0
         self.log_size = 0
         self.accept_sum = 0
@@ -311,7 +323,7 @@ class _Tree(object):
     def _single_step(self, left, epsilon):
         """Perform a leapfrog step and handle error cases."""
         try:
-            right = self.leapfrog(left.q, left.p, left.q_grad, epsilon)
+            right = self.integrator.step(epsilon, left)
         except linalg.LinAlgError as err:
             error_msg = "LinAlgError during leapfrog step."
             error = err
@@ -324,7 +336,6 @@ class _Tree(object):
             else:
                 raise
         else:
-            right = Edge(*right)
             energy_change = right.energy - self.start_energy
             if np.isnan(energy_change):
                 energy_change = np.inf
@@ -334,7 +345,7 @@ class _Tree(object):
             if np.abs(energy_change) < self.Emax:
                 p_accept = min(1, np.exp(-energy_change))
                 log_size = -energy_change
-                proposal = Proposal(right.q, right.energy, p_accept)
+                proposal = Proposal(right.q, right.q_grad, right.energy, p_accept)
                 tree = Subtree(right, right, right.p, proposal, log_size, p_accept, 1)
                 return tree, False, False
             else:
@@ -459,19 +470,20 @@ class NutsReport(object):
         """Print warnings for obviously problematic chains."""
         self._chain_id = strace.chain
 
-        tuning = strace.get_sampler_stats('tune')
-        if tuning.ndim == 2:
-            tuning = np.any(tuning, axis=-1)
+        if strace.supports_sampler_stats:
+            tuning = strace.get_sampler_stats('tune')
+            if tuning.ndim == 2:
+                tuning = np.any(tuning, axis=-1)
 
-        accept = strace.get_sampler_stats('mean_tree_accept')
-        if accept.ndim == 2:
-            accept = np.mean(accept, axis=-1)
+            accept = strace.get_sampler_stats('mean_tree_accept')
+            if accept.ndim == 2:
+                accept = np.mean(accept, axis=-1)
 
-        depth = strace.get_sampler_stats('depth')
-        if depth.ndim == 2:
-            depth = np.max(depth, axis=-1)
+            depth = strace.get_sampler_stats('depth')
+            if depth.ndim == 2:
+                depth = np.max(depth, axis=-1)
 
-        self._check_len(tuning)
-        self._check_depth(depth[~tuning])
-        self._check_accept(accept[~tuning])
-        self._check_divergence()
+            self._check_len(tuning)
+            self._check_depth(depth[~tuning])
+            self._check_accept(accept[~tuning])
+            self._check_divergence()
