@@ -6,7 +6,7 @@ import pymc3 as pm
 from .checks import close_to
 
 from .models import multidimensional_model, simple_categorical
-from ..plots import traceplot, forestplot, autocorrplot, plot_posterior
+from ..plots import traceplot, forestplot, autocorrplot, plot_posterior, energyplot
 from ..plots.utils import make_2d
 from ..step_methods import Slice, Metropolis
 from ..sampling import sample
@@ -25,10 +25,20 @@ def test_plots():
         step = Metropolis(model.vars, h)
         trace = sample(3000, tune=0, step=step, start=start)
 
-        traceplot(trace)
-        forestplot(trace)
-        plot_posterior(trace)
-        autocorrplot(trace)
+    traceplot(trace)
+    forestplot(trace)
+    plot_posterior(trace)
+    autocorrplot(trace)
+    energyplot(trace)
+
+
+def test_energyplot():
+    with asmod.build_model():
+        trace = sample()
+
+    energyplot(trace)
+    energyplot(trace, shade=0.5, alpha=0)
+    energyplot(trace, kind='hist')
 
 
 def test_plots_categorical():
@@ -92,5 +102,5 @@ def test_plots_transformed():
     assert traceplot(trace, plot_transformed=True).shape == (2, 2)
     assert autocorrplot(trace).shape == (1, 1)
     assert autocorrplot(trace, plot_transformed=True).shape == (2, 1)
-    assert plot_posterior(trace).shape == (1, )
+    assert plot_posterior(trace).numCols == 1
     assert plot_posterior(trace, plot_transformed=True).shape == (2, )
