@@ -2,7 +2,7 @@ from .checks import close_to
 import numpy as np
 from pymc3.tuning import starting
 from pymc3 import Model, Uniform, Normal, Beta, Binomial, find_MAP, Point
-from .models import simple_model, non_normal, exponential_beta, simple_arbitrary_det
+from .models import simple_model, non_normal, simple_arbitrary_det
 from .helpers import select_by_precision
 
 
@@ -20,19 +20,6 @@ def test_accuracy_non_normal():
         close_to(newstart['x'], mu, select_by_precision(float64=1e-5, float32=1E-4))
 
 
-def test_errors():
-    _, model, _ = exponential_beta(2)
-    with model:
-        try:
-            newstart = find_MAP(Point(x=[-.5, .01], y=[.5, 4.4]))
-        except ValueError as e:
-            msg = str(e)
-            assert "x.logp" in msg, msg
-            assert "x.value" not in msg, msg
-        else:
-            assert False, newstart
-
-
 def test_find_MAP_discrete():
     tol = 2.0**-11
     alpha = 4
@@ -41,8 +28,8 @@ def test_find_MAP_discrete():
     yes = 15
 
     with Model() as model:
-        p = Beta('p', alpha, beta, transform=None)
-        Binomial('ss', n=n, p=p, transform=None)
+        p = Beta('p', alpha, beta)
+        Binomial('ss', n=n, p=p)
         Binomial('s', n=n, p=p, observed=yes)
 
         map_est1 = starting.find_MAP()
@@ -68,14 +55,14 @@ def test_find_MAP():
     data = (data - np.mean(data)) / np.std(data)
 
     with Model():
-        mu = Uniform('mu', -1, 1, transform=None)
-        sigma = Uniform('sigma', .5, 1.5, transform=None)
+        mu = Uniform('mu', -1, 1)
+        sigma = Uniform('sigma', .5, 1.5)
         Normal('y', mu=mu, tau=sigma**-2, observed=data)
 
         # Test gradient minimization
         map_est1 = starting.find_MAP()
         # Test non-gradient minimization
-        map_est2 = starting.find_MAP(fmin=starting.optimize.fmin_powell)
+        map_est2 = starting.find_MAP(method="Powell")
 
     close_to(map_est1['mu'], 0, tol)
     close_to(map_est1['sigma'], 1, tol)
