@@ -1,11 +1,11 @@
 import shutil
 import tempfile
-import warnings
 
 from .checks import close_to
-from .models import simple_categorical, mv_simple, mv_simple_discrete, simple_2model, mv_prior_simple
+from .models import (simple_categorical, mv_simple, mv_simple_discrete,
+                     mv_prior_simple, simple_2model_continuous)
 from pymc3.sampling import assign_step_methods, sample
-from pymc3.model import Model, Deterministic
+from pymc3.model import Model
 from pymc3.step_methods import (NUTS, BinaryGibbsMetropolis, CategoricalGibbsMetropolis,
                                 Metropolis, Slice, CompoundStep, NormalProposal,
                                 MultivariateNormalProposal, HamiltonianMC,
@@ -27,26 +27,40 @@ from .helpers import select_by_precision
 class TestStepMethods(object):  # yield test doesn't work subclassing object
     master_samples = {
         Slice: np.array([
-            -8.13087389e-01, -3.08921856e-01, -6.79377098e-01, 6.50812585e-01, -7.63577596e-01,
-            -8.13199793e-01, -1.63823548e+00, -7.03863676e-02, 2.05107771e+00, 1.68598170e+00,
-            6.92463695e-01, -7.75120766e-01, -1.62296463e+00, 3.59722423e-01, -2.31421712e-01,
-            -7.80686956e-02, -6.05860731e-01, -1.13000202e-01, 1.55675942e-01, -6.78527612e-01,
-            6.31052333e-01, 6.09012517e-01, -1.56621643e+00, 5.04330883e-01, 3.14824082e-03,
-            -1.31287073e+00, 4.10706927e-01, 8.93815792e-01, 8.19317020e-01, 3.71900919e-01,
-            -2.62067312e+00, -3.47616592e+00, 1.50335041e+00, -1.05993351e+00, 2.41571723e-01,
-            -1.06258156e+00, 5.87999429e-01, -1.78480091e-01, -3.60278680e-01, 1.90615274e-01,
-            -1.24399204e-01, 4.03845589e-01, -1.47797573e-01, 7.90445804e-01, -1.21043819e+00,
-            -1.33964776e+00, 1.36366329e+00, -7.50175388e-01, 9.25241839e-01, -4.17493767e-01,
-            1.85311339e+00, -2.49715343e+00, -3.18571692e-01, -1.49099668e+00, -2.62079621e-01,
-            -5.82376852e-01, -2.53033395e+00, 2.07580503e+00, -9.82615856e-01, 6.00517782e-01,
-            -9.83941620e-01, -1.59014118e+00, -1.83931394e-03, -4.71163466e-01, 1.90073737e+00,
-            -2.08929125e-01, -6.98388847e-01, 1.64502092e+00, -1.19525944e+00, 1.44424109e+00,
-            1.52974876e+00, -5.70140077e-01, 5.08633322e-01, -1.70862492e-02, -1.69887948e-01,
-            5.19760297e-01, -4.15149647e-01, 8.63685174e-02, -3.66805233e-01, -9.24988952e-01,
-            2.33307122e+00, -2.60391496e-01, -5.86271814e-01, -5.01297170e-01, -1.53866195e+00,
-            5.71285373e-01, -1.30571830e+00, 8.59587795e-01, 6.72170694e-01, 9.12433943e-01,
-            7.04959179e-01, 8.37863464e-01, -5.24200836e-01, 1.28261340e+00, 9.08774240e-01,
-            8.80566763e-01, 7.82911967e-01, 8.01843432e-01, 7.09251098e-01, 5.73803618e-01]),
+            -5.95252353e-01, -1.81894861e-01, -4.98211488e-01,
+            -1.02262800e-01, -4.26726030e-01, 1.75446860e+00,
+            -1.30022548e+00, 8.35658004e-01, 8.95879638e-01,
+            -8.85214481e-01, -6.63530918e-01, -8.39303080e-01,
+            9.42792225e-01, 9.03554344e-01, 8.45254684e-01,
+            -1.43299803e+00, 9.04897201e-01, -1.74303131e-01,
+            -6.38611581e-01, 1.50013968e+00, 1.06864438e+00,
+            -4.80484421e-01, -7.52199709e-01, 1.95067495e+00,
+            -3.67960104e+00, 2.49291588e+00, -2.11039152e+00,
+            1.61674758e-01, -1.59564182e-01, 2.19089873e-01,
+            1.88643940e+00, 4.04098154e-01, -4.59352326e-01,
+            -9.06370675e-01, 5.42817654e-01, 6.99040611e-03,
+            1.66396391e-01, -4.74549281e-01, 8.19064437e-02,
+            1.69689952e+00, -1.62667304e+00, 1.61295808e+00,
+            1.30099144e+00, -5.46722750e-01, -7.87745494e-01,
+            7.91027521e-01, -2.35706976e-02, 1.68824376e+00,
+            7.10566880e-01, -7.23551374e-01, 8.85613069e-01,
+            -1.27300146e+00, 1.80274430e+00, 9.34266276e-01,
+            2.40427061e+00, -1.85132552e-01, 4.47234196e-01,
+            -9.81894859e-01, -2.83399706e-01, 1.84717533e+00,
+            -1.58593284e+00, 3.18027270e-02, 1.40566006e+00,
+            -9.45758714e-01, 1.18813188e-01, -1.19938604e+00,
+            -8.26038466e-01, 5.03469984e-01, -4.72742758e-01,
+            2.27820946e-01, -1.02608915e-03, -6.02507158e-01,
+            7.72739682e-01, 7.16064505e-01, -1.63693490e+00,
+            -3.97161966e-01, 1.17147944e+00, -2.87796982e+00,
+            -1.59533297e+00, 6.73096114e-01, -3.34397247e-01,
+            1.22357427e-01, -4.57299104e-02, 1.32005771e+00,
+            -1.29910645e+00, 8.16168850e-01, -1.47357594e+00,
+            1.34688446e+00, 1.06377551e+00, 4.34296696e-02,
+            8.23143354e-01, 8.40906324e-01, 1.88596864e+00,
+            5.77120694e-01, 2.71732927e-01, -1.36217979e+00,
+            2.41488213e+00, 4.68298379e-01, 4.86342250e-01,
+            -8.43949966e-01]),
         HamiltonianMC: np.array([
            -0.74925631, -0.2566773 , -2.12480977,  1.64328926, -1.39315913,
             2.04200003,  0.00706711,  0.34240498,  0.44276674, -0.21368043,
@@ -110,26 +124,47 @@ class TestStepMethods(object):  # yield test doesn't work subclassing object
               -3.91174647e-01,  -2.60664979e+00,  -2.27637534e+00,  -2.81505065e+00,
               -2.24238542e+00,  -1.01648100e+00,  -1.01648100e+00,  -7.60912865e-01,
                1.44384812e+00,   2.07355127e+00,   1.91390340e+00,   1.66559696e+00]),
-        smc.SMC: np.array([-0.94129179, -0.24527269, -0.21178402,  0.41638589,  0.98654812,
-                           -1.98046225,  1.12342667,  1.6415898 , -1.1200565 ,  0.32953477,
-                            1.39889789, -1.73171544,  0.0443656 ,  0.34281177, -0.56480463,
-                           -0.73692101, -0.51636731, -0.03384773, -1.38884227,  1.19531008,
-                            0.11847074,  0.48935035,  0.19281622, -0.29963799,  1.22910538,
-                            0.7410759 , -1.19513787,  1.13016882,  0.0564043 , -0.61605194,
-                           -0.0316803 ,  1.05364925,  0.69909548, -0.93454091,  0.52057634,
-                            0.56511333,  0.68718385, -0.17977616,  0.0465673 , -0.17018186,
-                            0.07703077,  0.75364315,  0.1199398 ,  0.99348863,  0.73182432,
-                            1.01110192,  0.07782113,  0.15190815, -0.2146857 ,  0.70445444,
-                            0.09726742,  0.47132485, -1.03145464,  0.44136444,  0.17668074,
-                           -0.42707508, -0.50563966, -0.39789354, -0.47026749,  0.35764632,
-                           -1.12201873, -0.44234106, -0.23588142,  0.94064406, -0.29664115,
-                            0.50927203,  0.12289724,  0.37487975,  0.69245379,  0.93506873,
-                           -0.38205722,  0.69079024,  0.15645901, -0.40054163, -0.56146871,
-                            0.08198217,  1.33145711, -0.28730786,  0.97928876, -0.11001317,
-                           -0.3198016 ,  0.29951594, -0.19081766,  0.50318481, -0.20971093,
-                           -0.26810202, -0.73819079,  0.5627611 , -0.48295116, -0.1490258 ,
-                           -0.25184255, -0.42765883,  1.45981846,  0.36949272,  1.1110567 ,
-                            0.4984653 , -0.13902672,  0.18949919, -0.54360687, -0.2740874 ]),
+        smc.SMC: np.array(
+        [ 1.30059573, -1.        ,  1.30059573, -1.28860918,  1.30059573,
+       -1.33854363,  0.98809372, -0.22700433,  0.98809372,  0.45421367,
+        0.53534095, -0.0571964 ,  0.53534095, -0.58075355,  0.53534095,
+       -0.81941713,  0.21067768, -0.77333386,  0.21067768, -0.57169475,
+        0.21067768, -0.57169475,  0.71695573, -0.1735022 ,  0.87048219,
+       -0.28469019,  1.08731483,  0.08746968,  1.08059419,  0.08746968,
+        0.31491769, -0.17753158,  0.48834878,  0.99152949, -0.1423678 ,
+        0.1923664 ,  0.06791856, -0.99708314, -0.13981681, -0.99708314,
+        0.15039906, -0.99708314,  0.15039906, -0.6557885 ,  0.15039906,
+       -0.6557885 ,  0.3553436 , -0.22781864,  0.3553436 , -0.86087058,
+        0.3553436 , -1.26758014,  0.3553436 , -0.02546953,  1.992939  ,
+        0.03739508,  1.992939  ,  0.04077929,  1.47964467, -0.79954537,
+        1.36470456, -1.28038148,  1.34975939, -1.28038148,  0.5058148 ,
+       -1.28038148,  0.46681777, -1.51635697,  1.14761057, -1.51635697,
+        0.70585017, -1.51635697,  0.56298035, -1.51635697,  0.68107999,
+       -1.24900543,  0.68107999, -1.58687463,  0.8251361 , -0.30236423,
+        0.19971902,  0.0871776 ,  0.19971902,  0.51328569,  0.19971902,
+        0.55526923,  0.19971902,  0.16065882, -0.87573391,  0.42539449,
+       -0.87573391,  0.31060689, -0.87573391,  0.48370178,  0.27495794,
+        0.48370178,  0.37129344,  0.48370178,  1.06413954,  0.48370178,
+        1.57177313,  0.01683961,  1.75583481,  0.01683961,  1.87895941,
+        0.49419352,  1.87895941,  0.49419352,  1.58832631, -0.02168877,
+        1.58832631,  0.79617759,  1.41454982,  0.79617759,  1.93168471,
+        0.78016131,  1.73345978,  0.19202933,  1.62254723, -0.22699057,
+        1.62254723, -0.37699978, -0.14380698, -1.39915323, -0.0647066 ,
+       -1.39915323, -0.27796904, -0.76046542, -1.39097353, -0.95882837,
+       -2.37809137, -0.95882837, -0.76288136, -0.95882837,  0.10702519,
+        0.6982565 ,  0.47017639,  0.6982565 ,  0.9479599 ,  1.22950397,
+        0.9479599 ,  0.06416429,  0.9479599 , -0.0761023 ,  1.18090459,
+       -0.54169371,  1.18090459,  0.00447742,  0.93159018,  0.77757319,
+        0.76557639,  0.77757319,  0.41731133,  0.77757319,  0.68380287,
+        1.08320749, -0.43989818,  1.05260977, -0.32229488,  1.15892126,
+        0.38764854,  1.15892126,  0.41491972,  0.71650402, -0.27923606,
+        0.71650402, -1.32493526, -0.09371072, -0.74606271,  0.5227973 ,
+       -0.74606271,  0.63974633, -1.41947892, -0.96488174, -1.294319  ,
+       -0.96488174, -1.294319  , -0.78112189, -1.294319  ,  0.45821163,
+       -1.31499922,  0.54901984, -1.10260234,  1.12896946,  0.43768361,
+        1.12896946, -0.58455279,  1.12896946, -0.58455279,  1.12896946,
+       -0.58455279,  0.19820143, -1.19295628, -0.02548627, -1.19295628,
+        0.50411866, -1.19295628,  0.50411866, -1.55631463,  0.92268245]),
     }
 
     def setup_class(self):
@@ -160,18 +195,25 @@ class TestStepMethods(object):  # yield test doesn't work subclassing object
         on multiple commits.
         """
         n_steps = 100
-        with Model():
+        with Model() as model:
             x = Normal('x', mu=0, sd=1)
             if step_method.__name__ == 'SMC':
-                Deterministic('like', - 0.5 * tt.log(2 * np.pi) - 0.5 * x.T.dot(x))
-                trace = smc.ATMIP_sample(n_steps=n_steps, step=step_method(random_seed=1),
-                                         n_jobs=1, progressbar=False,
-                                         homepath=self.temp_dir)
+                trace = smc.sample_smc(n_steps=n_steps,
+                                       n_chains=2,
+                                       start=[{'x':1.}, {'x':-1.}],
+                                       random_seed=1,
+                                       n_jobs=1, progressbar=False,
+                                       homepath=self.temp_dir)
+
+            elif step_method.__name__ == 'NUTS':
+                step = step_method(scaling=model.test_point)
+                trace = sample(0, tune=n_steps,
+                               discard_tuned_samples=False,
+                               step=step, random_seed=1)
             else:
                 trace = sample(0, tune=n_steps,
                                discard_tuned_samples=False,
                                step=step_method(), random_seed=1)
-
         assert_array_almost_equal(
             trace.get_values('x'),
             self.master_samples[step_method],
@@ -280,7 +322,7 @@ class TestCompoundStep(object):
                         reason="Test fails on 32 bit due to linalg issues")
     def test_non_blocked(self):
         """Test that samplers correctly create non-blocked compound steps."""
-        _, model = simple_2model()
+        _, model = simple_2model_continuous()
         with model:
             for sampler in self.samplers:
                 assert isinstance(sampler(blocked=False), CompoundStep)
@@ -288,7 +330,7 @@ class TestCompoundStep(object):
     @pytest.mark.skipif(theano.config.floatX == "float32",
                         reason="Test fails on 32 bit due to linalg issues")
     def test_blocked(self):
-        _, model = simple_2model()
+        _, model = simple_2model_continuous()
         with model:
             for sampler in self.samplers:
                 sampler_instance = sampler(blocked=True)
@@ -334,9 +376,9 @@ class TestAssignStepMethods(object):
 class TestNutsCheckTrace(object):
     def test_multiple_samplers(self):
         with Model():
-            prob = Beta('prob', alpha=5, beta=3)
+            prob = Beta('prob', alpha=5., beta=3.)
             Binomial('outcome', n=1, p=prob)
-            with warnings.catch_warnings(record=True) as warns:
+            with pytest.warns(None) as warns:
                 sample(3, tune=2, discard_tuned_samples=False,
                        n_init=None)
             messages = [warn.message.args[0] for warn in warns]
@@ -356,12 +398,15 @@ class TestNutsCheckTrace(object):
             a = tt.switch(a > 0, np.inf, a)
             b = tt.slinalg.solve(floatX(np.eye(2)), a)
             Normal('c', mu=b, shape=2)
-            with warnings.catch_warnings(record=True) as warns:
+            with pytest.warns(None) as warns:
                 trace = sample(20, init=None, tune=5)
+            warns = [str(warn.message) for warn in warns]
             assert np.any(trace['diverging'])
-            assert any('diverging samples after tuning' in str(warn.message)
+            assert any('diverging samples after tuning' in warn
                        for warn in warns)
-            assert any('contains only' in str(warn.message) for warn in warns)
+            # FIXME This test fails sporadically on py27.
+            # It seems that capturing warnings doesn't work as expected.
+            # assert any('contains only' in warn for warn in warns)
 
             with pytest.raises(SamplingError):
                 sample(20, init=None, nuts_kwargs={'on_error': 'raise'})
