@@ -25,7 +25,10 @@ from .inference import (
     FullRankADVI,
     SVGD,
     ASVGD,
+    NFVI,
     Inference,
+    KLqp,
+    ImplicitGradient,
     fit
 )
 
@@ -34,12 +37,18 @@ from .approximations import (
     MeanField,
     FullRank,
     Empirical,
+    NormalizingFlow,
     sample_approx
 )
+from . import opvi
+from .opvi import (
+    Group,
+    Approximation
+)
 
 # special
 from .stein import Stein
+from . import flows
 from . import operators
 from . import test_functions
-from . import opvi
 from . import callbacks
