@@ -2,7 +2,7 @@
 '''
 (c) 2016, John Salvatier & Taku Yoshioka
 '''
-from collections import OrderedDict, namedtuple
+from collections import OrderedDict, namedtuple, deque
 
 import numpy as np
 import theano
@@ -11,6 +11,7 @@ from theano.sandbox.rng_mrg import MRG_RandomStreams
 
 import pymc3 as pm
 from pymc3.backends.base import MultiTrace
+from ..theanof import floatX
 
 from tqdm import trange
 
@@ -19,14 +20,6 @@ __all__ = ['advi', 'sample_vp']
 ADVIFit = namedtuple('ADVIFit', 'means, stds, elbo_vals')
 
 
-def check_discrete_rvs(vars):
-    """Check that vars not include discrete variables, excepting ObservedRVs.
-    """
-    vars_ = [var for var in vars if not isinstance(var, pm.model.ObservedRV)]
-    if any([var.dtype in pm.discrete_types for var in vars_]):
-        raise ValueError('Model should not include discrete RVs for ADVI.')
-
-
 def gen_random_state():
     """Helper to generate a random state for MRG_RandomStreams"""
     M1 = 2147483647
@@ -36,7 +29,8 @@ def gen_random_state():
 
 
 def advi(vars=None, start=None, model=None, n=5000, accurate_elbo=False,
-         optimizer=None, learning_rate=.001, epsilon=.1, random_seed=None):
+         optimizer=None, learning_rate=.001, epsilon=.1, mode=None,
+         tol_obj=0.01, eval_elbo=100, random_seed=None, progressbar=True):
     """Perform automatic differentiation variational inference (ADVI).
 
     This function implements the meanfield ADVI, where the variational
@@ -85,8 +79,20 @@ def advi(vars=None, start=None, model=None, n=5000, accurate_elbo=False,
     epsilon : float
         Offset in denominator of the scale of learning rate in Adagrad.
         This parameter is ignored when optimizer is given.
+    tol_obj : float
+        Relative tolerance for testing convergence of ELBO.
+    eval_elbo : int
+        Window for checking convergence of ELBO. Convergence will be checked
+        for every multiple of eval_elbo.
     random_seed : int or None
         Seed to initialize random state. None uses current seed.
+    mode :  string or `Mode` instance.
+        Compilation mode passed to Theano functions
+    progressbar : bool
+        Whether or not to display a progress bar in the command line. The
+        bar shows the percentage of completion, the sampling speed in
+        samples per second (SPS), the estimated remaining time until
+        completion ("expected time of arrival"; ETA), and the current ELBO.
 
     Returns
     -------
@@ -102,6 +108,10 @@ def advi(vars=None, start=None, model=None, n=5000, accurate_elbo=False,
         and Blei, D. M. (2016). Automatic Differentiation Variational
         Inference. arXiv preprint arXiv:1603.00788.
     """
+    import warnings
+    warnings.warn('Old ADVI interface and sample_vp is deprecated and will '
+                  'be removed in future, use pm.fit and pm.sample_approx instead',
+                  DeprecationWarning, stacklevel=2)
     model = pm.modelcontext(model)
     if start is None:
         start = model.test_point
@@ -110,8 +120,11 @@ def advi(vars=None, start=None, model=None, n=5000, accurate_elbo=False,
         vars = model.vars
     vars = pm.inputvars(vars)
 
+    if len(vars) == 0:
+        raise ValueError('No free random variables to fit.')
+
     if not pm.model.all_continuous(vars):
-        raise ValueError('Model should not include discrete RVs for ADVI.')
+        raise ValueError('Model can not include discrete RVs for ADVI.')
 
     n_mcsamples = 100 if accurate_elbo else 1
 
@@ -137,26 +150,77 @@ def advi(vars=None, start=None, model=None, n=5000, accurate_elbo=False,
     uw_shared = theano.shared(uw, 'uw_shared')
     elbo = pm.CallableTensor(elbo)(uw_shared)
     updates = optimizer(loss=-1 * elbo, param=[uw_shared])
-    f = theano.function([], [uw_shared, elbo], updates=updates)
+    f = theano.function([], [uw_shared, elbo], updates=updates, mode=mode)
+
+    # For tracking convergence of ELBO
+    window_size = int(max(0.1 * n // eval_elbo, 2.0))
+    circ_buff = deque([], maxlen=window_size)
 
     # Optimization loop
     elbos = np.empty(n)
+    divergence_flag = False
+    progress = trange(n) if progressbar else range(n)
     try:
-        progress = trange(n)
+        uw_i, elbo_current = f()
+        if np.isnan(elbo_current):
+            raise FloatingPointError('NaN occurred in ADVI optimization.')
         for i in progress:
             uw_i, e = f()
+            if np.isnan(e):
+                raise FloatingPointError('NaN occurred in ADVI optimization.')
             elbos[i] = e
-            if i % (n // 10) == 0 and i > 0:
-                avg_elbo = elbos[i - n // 10:i].mean()
-                progress.set_description('Average ELBO = {:,.5g}'.format(avg_elbo))
+
+            if progressbar:
+                if n < 10:
+                    progress.set_description('ELBO = {:,.5g}'.format(elbos[i]))
+                elif i % (n // 10) == 0 and i > 0:
+                    avg_elbo = infmean(elbos[i - n // 10:i])
+                    progress.set_description(
+                        'Average ELBO = {:,.5g}'.format(avg_elbo))
+
+            if i % eval_elbo == 0:
+                elbo_prev = elbo_current
+                elbo_current = elbos[i]
+                delta_elbo = abs((elbo_current - elbo_prev) / elbo_prev)
+                circ_buff.append(delta_elbo)
+                avg_delta = np.mean(circ_buff)
+                med_delta = np.median(circ_buff)
+
+                if i > 0 and avg_delta < tol_obj:
+                    pm._log.info('Mean ELBO converged.')
+                    elbos = elbos[:(i + 1)]
+                    break
+                elif i > 0 and med_delta < tol_obj:
+                    pm._log.info('Median ELBO converged.')
+                    elbos = elbos[:(i + 1)]
+                    break
+                if i > 10 * eval_elbo:
+                    if med_delta > 0.5 or avg_delta > 0.5:
+                        divergence_flag = True
+                    else:
+                        divergence_flag = False
+
     except KeyboardInterrupt:
         elbos = elbos[:i]
-        avg_elbo = elbos[i - n // 10:].mean()
-        pm._log.info('Interrupted at {:,d} [{:.0f}%]: Average ELBO = {:,.5g}'.format(
-            i, 100 * i // n, avg_elbo))
+        if n < 10:
+            pm._log.info('Interrupted at {:,d} [{:.0f}%]: ELBO = {:,.5g}'.format(
+                i, 100 * i // n, elbos[i]))
+        else:
+            avg_elbo = infmean(elbos[i - n // 10:i])
+            pm._log.info('Interrupted at {:,d} [{:.0f}%]: Average ELBO = {:,.5g}'.format(
+                i, 100 * i // n, avg_elbo))
     else:
-        avg_elbo = elbos[-n // 10:].mean()
-        pm._log.info('Finished [100%]: Average ELBO = {:,.5g}'.format(avg_elbo))
+        if n < 10:
+            pm._log.info('Finished [100%]: ELBO = {:,.5g}'.format(elbos[-1]))
+        else:
+            avg_elbo = infmean(elbos[-n // 10:])
+            pm._log.info('Finished [100%]: Average ELBO = {:,.5g}'.format(avg_elbo))
+    finally:
+        if progressbar:
+            progress.close()
+
+    if divergence_flag:
+        pm._log.info('Evidence of divergence detected, inspect ELBO.')
 
     # Estimated parameters
     l = int(uw_i.size / 2)
@@ -181,8 +245,8 @@ def _calc_elbo(vars, model, n_mcsamples, random_seed):
     [logp], inarray = pm.join_nonshared_inputs([logpt], vars, shared)
 
     uw = tt.vector('uw')
-    uw.tag.test_value = np.concatenate([inarray.tag.test_value,
-                                        inarray.tag.test_value]).astype(theano.config.floatX)
+    uw.tag.test_value = floatX(np.concatenate([inarray.tag.test_value,
+                                               inarray.tag.test_value]))
 
     elbo = _elbo_t(logp, uw, inarray, n_mcsamples, random_seed)
 
@@ -193,7 +257,7 @@ def _elbo_t(logp, uw, inarray, n_mcsamples, random_seed):
     """Create Theano tensor of approximate ELBO by Monte Carlo sampling.
     """
     l = (uw.size / 2)
-    l_int = l.astype('int64')
+    l_int = l.astype('int32')
     u = uw[:l_int]
     w = uw[l_int:]
 
@@ -251,8 +315,8 @@ def adagrad_optimizer(learning_rate, epsilon, n_win=10):
             param = list(param)
 
         for param_ in param:
-            i = theano.shared(np.array(0, dtype=theano.config.floatX))
-            i_int = i.astype('int64')
+            i = theano.shared(floatX(np.array(0)))
+            i_int = i.astype('int32')
             value = param_.get_value(borrow=True)
             accu = theano.shared(
                 np.zeros(value.shape + (n_win,), dtype=value.dtype))
@@ -276,7 +340,7 @@ def adagrad_optimizer(learning_rate, epsilon, n_win=10):
 
 def sample_vp(
         vparams, draws=1000, model=None, local_RVs=None, random_seed=None,
-        hide_transformed=True, progressbar=True):
+        include_transformed=False, progressbar=True):
     """Draw samples from variational posterior.
 
     Parameters
@@ -289,14 +353,18 @@ def sample_vp(
         Probabilistic model.
     random_seed : int or None
         Seed of random number generator.  None to use current seed.
-    hide_transformed : bool
-        If False, transformed variables are also sampled. Default is True.
+    include_transformed : bool
+        If True, transformed variables are also sampled. Default is False.
 
     Returns
     -------
     trace : pymc3.backends.base.MultiTrace
         Samples drawn from the variational posterior.
     """
+    import warnings
+    warnings.warn('Old ADVI interface and sample_vp is deprecated and will '
+                  'be removed in future, use pm.fit and pm.sample_approx instead',
+                  DeprecationWarning, stacklevel=2)
     model = pm.modelcontext(model)
 
     if isinstance(vparams, ADVIFit):
@@ -342,11 +410,8 @@ def sample_vp(
     f = theano.function([], samples)
 
     # Random variables which will be sampled
-    if hide_transformed:
-        vars_sampled = [v_ for v_ in model.unobserved_RVs
-                        if not str(v_).endswith('_')]
-    else:
-        vars_sampled = [v_ for v_ in model.unobserved_RVs]
+    vars_sampled = pm.util.get_default_varnames(model.unobserved_RVs,
+                                                include_transformed=include_transformed)
 
     varnames = [str(var) for var in model.unobserved_RVs]
     trace = pm.sampling.NDArray(model=model, vars=vars_sampled)
@@ -360,3 +425,8 @@ def sample_vp(
         trace.record(point)
 
     return MultiTrace([trace])
+
+
+def infmean(input_array):
+    """Return the mean of the finite values of the array"""
+    return np.mean(np.asarray(input_array)[np.isfinite(input_array)])
